c    written by Paolo  Coppi 
c    changed by Juri to increase the speed (Oct 28, 1997) 
c    relativistic reflection is added Oct 29, 1997
c    parameters changed by MG on Nov 5, 1997
c    diskbb replaced by diskpn by MG, Jun 28, 1998
c    fits data load routines added by PC, Jul 23, 2001

      SUBROUTINE XSCOMPTH(Ear,Ne,Param,Ifl,Photar,Photer)
      include "common.eq"
      common /normdisk/ radnorm

      INTEGER Ne , Ifl
      REAL*4 Param(20) , Ear(0:Ne) , Photar(Ne), Photer(Ne)

c
c     driver for the nonthermal/thermal pair program
c
c     number of model parameters: 18
c     parameters are given below in the order of decreasing importance:
c     1: hard-to-soft compactness ratio
c     2: blackbody compactness (if <0, uncomptonized black body component only)
c     3: blackbody temperature in eV (if <0, use diskpn, T_{max}=abs(Par(3)) )
c     4: non-thermal-to-hard compactness ratio (0 for pure thermal plasma)
c     5: Thomson optical depth of ionization electrons (e.g., 0)
c     6: source region radius in cm (for Coulomb/bremsstrahlung only)
c     7: minimum Lorentz factor for power law injection (not used for mono)
c     8: maximum Lorentz factor for power law injection 
c     9: power law injection index (if <0, inject monoenergetically at #8)
c     10 choose primary pair or electron injection (0=electron, 1=pair)
c     11: cosIncl, cosine of inclination angle
c     12, R, amount of reflection (if < 0 then only reflection component)
c     13: FeAb, iron abundance in units of solar
c     14: abundance of elements heavier than He relative to solar
c     15: disk temperature (used for reflection)
c     16: xi, ionization parameter
c     17: beta, reflection emissivity law (r^beta) IF=-10 then 
c         non-rotating disk, IF=10 then  (1.-sqrt(6./rg))/rg**3
c     18: radin, inner radius of the disk in M
c     19: radout, outer radius of the disk in M
c     20: redshift
c
c    Note: if #3 OR #12 < 0, only show BB and/or Reflected components
c
c     algorithm:
c           a(x)=(non-thermal/thermal pair spectrum)+reflection
c

      INTEGER n , np ,  ierr, nem,ne1,neold 
      parameter (nem=5000)
      parameter (np=20,nref=9,nrefl=7,ndisk=2)
      REAL*4 pa0(np),parref(nref),par_refl(nrefl),RedShift,RefScale 
      REAL*4 par_disk(ndisk),photdisk(nem)
      REAL*4 p1, p4
      real*8 xg(20),wg(20),wkev(221),
     & phref(nem),photco(nem),photre(nem),obj(nem),objold(nem),   
     & phbb(nem),photbb(nem)
      common/compsp1/ ospect(221)

      real*4 tear(0:221)
 
      LOGICAL firstcall , fl0 , fl1 , fl2 
      CHARACTER contxt*127

      SAVE firstcall, fl0, fl1, fl2, pa0, parref
      SAVE xg,wg,neold,objold,RedShift,RefScale
      SAVE phref,photre,photco,phbb,photbb

      DATA firstcall/.TRUE./,fl0/.true./,fl1/.true./,fl2/.true./
      DATA pa0/np*999./,parref/nref*999./,RedShift/0./
      DATA neold/999/ 
      logical showbb


c change input XSPEC parameters:
c
c   p(1) = (l_th + l_nth) / l_bb
c   p(2) = l_bb
c   p(4) = l_nth / (l_th + l_nth)
c
c into new values:
c
c   p(1) = l_nth
c   p(2) = l_bb
c   p(4) = l_th

      if (Param(2) .lt. 0.0) then
        Param(2) = -Param(2) 
        showbb  = .true.
      else
        showbb = .false.
      endif

c      p1 = Param(1) * Param(2) * Param(4)
c      p4 = Param(1) * Param(2) * (1.0 - Param(4))
c      Param(1) = p1
c      Param(4) = p4

c      print *, 'theta = ', Param(1)
c      print *, 'l_th  = ', Param(4)

c object's frame energies (in mc2)
      ne1=ne+1
         do i=0,ne
            obj(i+1)=dble((1.+Param(20))*ear(i)/511.)
         enddo

      IF ( firstcall ) THEN
         CALL xwrite('     ',5)
         CALL xwrite('            CompTH V1.01, FITS data version',5)
         CALL xwrite('Still experimental  -- use at your own risk!',5)
         CALL xwrite('For questions, contact Paolo Coppi',5)
         CALL xwrite('at coppi@astro.yale.edu .',5)

         call gauleg(0.d0,1.d0,xg,wg,10)
      ENDIF


c  calculate internal source spectrum if necessary
cjp checking only for the first 10 parameters (forget about accuracy)
      DO 100 n = 1 , 10 
         IF ( Param(n).NE.pa0(n) ) fl0 = .TRUE.
 100  CONTINUE
c Check also R_in
      if (Param(18) .ne. pa0(18)) fl0 = .TRUE.
c      if (fl0) print *,'need to recompute model'
      IF (fl0.or.Param(20).ne.RedShift) THEN
         CALL thcpairiter(Param,firstcall,np)
          do i=1,221
c total emergent spectrum including black body
              esc(i) = esc(i)/gwidth(i)
c separate Comptonized continuum and bb!!!! 
c
              bbspect(i) = bbspect(i)/gwidth(i)
              ospect(i)=esc(i)-bbspect(i)
c ACHTUNG
c    can't allow zeros or things get all messed up 
c    (spect interp. takes log(0)-- oops)
              ospect(i) = max(1.d-30,ospect(i))
              bbspect(i) = max(1.d-30,bbspect(i))
              tspect(i) = max(1.d-30,tspect(i))
          enddo
c screws up near annihilation line!
c          call qspline(wkev,ospect,221,1d30,1d30,f2)

c renormalize to disk model
          fnorm = radnorm / 511.0

c      xn = (1.d0+Param(20))/511. 
c interpolate to get spectrum at 1 KeV
c        targ = xn
c        i = max(min(int(log10(targ)*20.d0+141.5d0),221),1)
c        if (targ.lt.wgam(i))  i=max(i-1,1)
c        i2 = min(221,i+1)
c        x0 = wgam(i)
c        x1 = wgam(i2)
c        y1 = ospect(i2)
c        y0 = ospect(i)
c        if (i.ne.i2) then 
c           tempeqp = y0*exp(log(y1/y0)/log(x1/x0)*log(targ/x0))
c        else
c           tempeqp = y0
c        endif
cjp normalizing only continuum radiation without reflection 
c        fnorm = 1.d0/tempeqp

          do i=1,221
            ospect(i) = ospect(i)*fnorm
          enddo

      ENDIF
 


cc ** Now add reflection!
c      
c check obj 
      if(neold.ne.ne) fl2=.true.
      do i=1,ne1 
      if(obj(i).ne.objold(i)) fl2=.true.
      enddo 
c check if reflection parameters have changed 
      do i=13,19 
      if(param(i).ne.parref(i-10)) then 
      fl1=.true. 
      endif 
      enddo 
      if(param(11).ne.parref(1)) fl1=.true.
      if(RedShift.ne.Param(20)) fl1=.true.

c reflection 
c normalization 
      RefScale=abs(Param(12)) 

      if (Param(4).gt. 0.d0) then 
c completely shut off reflection if negative

      IF((fl0.or.fl1.or.fl2).and.(RefScale.gt.1e-6.or.firstcall)) THEN 
cc ionized reflector (and possibly rotating) 
c  ab0, iron log10 abundance relative to solar
      par_refl(1)=alog10(Param(13)) 
c  ab_, log10 abundance of elements heavier than He relative to
c       the solar abundances
      par_refl(2)=alog10(Param(14))
c  xi, ionization parameter
      par_refl(3)=Param(16)
c  temp, disk temperature (for ionization balance, Tmax=10^6 K)
      par_refl(4)=Param(15)
c  beta      - reflection emissivity law (r^beta)
      par_refl(5)=Param(17)
c  radin     - inner radius of the disk
      par_refl(6)=Param(18)
c  radout    - outer radius of the disk
      par_refl(7)=Param(19)
c cos angle 
      angle = Param(11)
c     wgam - energies (221)
c   ospect  - angle averaged comptonized spectrum 
cjp ACHTUNG: here one should have ONLY comptonized spectrum
cjp NOT bb+comptonized !!!!!!!! 
c     obj - energies where refl should be computed
c     phref  - reflected spectrum at a given angle
      irefl=0
c relativistic disk reflector
      if(Param(17).ne.-10.) irefl=1 
      call comprefl1(wgam,ospect,obj,phref,angle,
     + par_refl,nrefl,221,ne1,irefl)
      ENDIF 
c   end of reflection on/off
      ENDIF

c    rebinning of the comptonized spectrum
      IF(fl0.or.fl2.or.Param(20).ne.RedShift) THEN 
         do i=1,221
           wkev(i) = wgam(i)*511d0
         enddo
      do it=0,Ne-1
        xlo = Ear(it)
        xhi = Ear(it+1)
        dx = xhi-xlo
        sum = 0.d0
        do j=1,10
          targ = (xg(j)*dx + xlo)*(1.d0+Param(20))
c--- inline expansion
          if ((targ.lt.wkev(1)).or.(targ.gt.wkev(221))) then 
            tempeqp = 1d-30
          else
            i = max(min(int(log10(targ/511.d0)*20.d0+141.5d0),221),1)
            if (targ.lt.wkev(i))  i=max(i-1,1)
            i2 = min(221,i+1)
            x0 = wkev(i)
            x1 = wkev(i2)
            y1 = ospect(i2)
            y0 = ospect(i)
            if (i.ne.i2) then 
              tempeqp= y0*exp(log(y1/y0)/log(x1/x0)*log(targ/x0))
            else
              tempeqp = y0
            endif
          endif
          sum = sum + tempeqp*wg(j)*dx
        enddo
        photco(it+1) = sum
      enddo

c    rebinning of the bb spectrum
      do it=0,Ne-1
        xlo = Ear(it)
        xhi = Ear(it+1)
        dx = xhi-xlo
        sum = 0.d0
        do j=1,10
          targ = (xg(j)*dx + xlo)*(1.d0+Param(20))
c--- inline expansion
          if ((targ.lt.wkev(1)).or.(targ.gt.wkev(221))) then 
            tempeqp = 1d-30
          else
            i = max(min(int(log10(targ/511.d0)*20.d0+141.5d0),221),1)
            if (targ.lt.wkev(i))  i=max(i-1,1)
            i2 = min(221,i+1)
            x0 = wkev(i)
            x1 = wkev(i2)
            y1 = bbspect(i2)
            y0 = bbspect(i)
            if (i.ne.i2) then 
              tempbb= y0*exp(log(y1/y0)/log(x1/x0)*log(targ/x0))
            else
              tempbb = y0
            endif
          endif
          sum = sum + tempbb*wg(j)*dx
        enddo
c ** DON'T FORGET normalization
        sum = sum * fnorm
        photbb(it+1) = sum
      enddo
      ENDIF 

c -    now get reflection component
      IF(RefScale.gt.1e-6) THEN
       IF(fl2.or.fl1.or.fl0.or.Param(12).ne.parref(2)) THEN 
c integrate over the bin size
       do i=1,Ne
       photre(i)=(phref(i+1)+phref(i))*dble((ear(i)-ear(i-1))/2.)
     &  *RefScale
       enddo
       ENDIF 
      ELSE 
      do i=1,Ne
      photre(i)=0. 
      enddo
      ENDIF 


      do i=1,Ne
        photar(i) = 0.0
        if ((Param(12).lt.0.).or.(showbb)) then
          if (Param(12).lt.0.) photar(i)=photre(i)+photar(i)
          if (showbb)          photar(i)=photbb(i)+photar(i)
        else
           photar(i)=sngl(photco(i)+photre(i)+photbb(i))
        endif
      enddo

         Do n = 1 , np
            pa0(n) = Param(n)
         EndDo
         fl0 = .FALSE.

         Do n = 11 , 19
            parref(n-10) = Param(n)
         EndDo
         RedShift=Param(20) 
         fl1 = .FALSE.

         Do i = 1, Ne1 
            objold(i)=obj(i)
         EndDo
         neold=ne 
         fl2 = .FALSE.

         firstcall = .FALSE.
c      print *,'o.k..finished w/eqpair'

      return
 
 
 999  CONTINUE
      IF ( ierr .NE. 0 ) THEN
         CALL xwrite(contxt, 10)
      ENDIF
 
      RETURN
      END

c ===================================


      subroutine thcpairiter(param,firstcall,np)

      include "common.eq"

      real*4 param(np)
      logical firstcall


c set model parameters and initialize model
      call thcsetup(param,firstcall,np)

c double check for bad input parameters
      if((rle .lt. 1.d-40).and.(rlth .lt. 1.d-40)
     &   .and.(taup.lt.1d-40)) then
c        do nothing!!  (no pairs!, gdist(i) = s(i) in setup)
         do i=1,ngambin
            esc(i) = gdist(i)
         enddo
         return
      endif
      if((rle .lt. 1.d-40).and.(rlth .gt. 1.d-40)
     &   .and.(taup.lt.1d-40)) then
         print *,'ILLEGAL INPUT COMBINATION:  l_th is not zero, yet you'
         print *,'have no background electrons/positrons since tau_p=0'
         print *,'and l_nth=0!!     '
         pause
         do i=1,ngambin
            esc(i) = gdist(i)
         enddo
         return
      endif


c get model spectrum
      call thcsolve

c get uncomptonized blackbody spectrum

      call bbesc

      return
      end





c ---------------------------
      subroutine thcdopcomp
      include "common.eq" 

c      print *,'o.k. bosses, theta,edist(1)',theta,edist(1)

      thresh = 1.d-5

c assume pout cleared, pin set to s(i)
c assume e- dist fixed


c call doesc + annihilation + brem + anything else that makes photons
       do i=1,ngambin
         gder(i) = 0.d0
       enddo
       call thcdotherm

       do i=1,ngambin
         pout(i) = pout(i) + tesc(i)
         pin(i) = pin(i) + gder(i)
         pin0(i) = pin(i)
       enddo

c       print *,'called dotherm'

       call mkscat
c       print *,'made scattering matrix'


      do i=1,ngambin
         pout0(i) = pout(i)
      enddo

c  now iterate


1     continue

       do i=1,ngambin-20
          do j=iscat(i,1),iscat(i,2)
             pin(j) = pin(j) + scat(i,j)*gdist(i)
          enddo
       enddo


      dmax = 0
      do i=1,ngambin
        temp = pin(i)/pout(i)
        odmax = dmax
        dmax = max(abs(temp-gdist(i))/(gdist(i)+1.d-40),dmax)
        if (dmax.ne.odmax) idmax = i
        gdist(i) = temp
      enddo

c      print *,'thresh?,dmax',thresh,dmax


      if (dmax .gt. thresh) Then
         do i=1,ngambin
           pout(i) = pout0(i)
           pin(i) = pin0(i)
         enddo
         goto 1
      endif


c      print *,'hello!!!'
      do i=1,ngambin
        esc(i) = tesc(i)*gdist(i)
c        print *,i,esc(i),tesc(i),gdist(i),pout(i),pin(i)
      enddo


c      print *,'RETURNING!!'
      return
      end


      subroutine thcsetup(param,firstcall,np)
      include "common.eq"
      common /normdisk/ radnorm

      logical firstcall
      real*4 param(np), tparam(2),tphotar(221),tear(0:221)

c      print *,'in setup' 
c      rle = param(1)
      rle = 0.d0
      rls = 1.d0
c electron temperature in m_ec^2
      theta = param(1)/5.11d2
c      print *,'theta now is',theta
c blackbody temperature in m_ec^2
      tr  = param(3)/5.11d5
      rlth = param(4)
      taup = param(5)
      rad  = param(6)

      Rin = param(18)

      igmax = max(min(int(log10(param(8))/.05d0+1.5d0),81),2)
      plawin = param(9)
      if (plawin .gt. 0.d0) then
        igmin = max(min(int(log10(param(7))/.05d0+1.5d0),81),2)
      endif
      if (param(10).gt.0) then
         pairinj = .true.
      else
         pairinj = .false.
      endif
      ians2 = 0
      b0 = 0.d0
      ians1 = 1
cJP is it OK? it should be param(18) but in fact it is always 0.02
      econv = param(18)
c      econv = param(16)

      maxiter = 10000

c      print *,'db: rle,rls,tr,rad',rle,rls,tr,rad
c      print *,'igmin,igmax,plawin',igmin,igmax,plawin
c      print *,'econv,maxiter',econv,maxiter

      nebin = 81
      ngambin = 221
        ioff = 40
        rioff1 = 141.5
        wmin = 10**(-7.025)
        wmax = 10.**4.025
c initial idump = bin# e- cascade into when using stationary approx
      idump = nebin
c initialize energy bin values
      do 58 i = 1,nebin
        temp = 10.**(float(i-1)*.05d0)
        een(i) = .5d0*temp*(10.**.025d0 + 10.**(-.025d0))
        upeen(i) = 10.**.025d0*temp
58    continue
      een(1) = (10.**.05d0 - 1.)/2./(10.**.025d0 - 1.)
      do 59 i = 2,nebin
59      ewidth(i) = upeen(i) - upeen(i-1)
      ewidth(1) = upeen(1) - 1.d0
c do photons
      do 62 i = 1,ngambin
        temp1 = 10.**(float(i-101-ioff)*.05d0)
        wgam(i) = .5d0*temp1*(10.**.025d0 + 10.**(-.025d0))
        upwgam(i)= 10.**.025d0*temp1
c        print *,i,wgam(i)
c        pause
62    continue
      do 60 i = 2,ngambin
        gwidth(i) = upwgam(i) - upwgam(i-1)
60    continue
      gwidth(1) = upwgam(1)-10.**(-7.025d0)
c
c  --- get model parameters
c      write(6,*) 'Hello...'
c      write(6,*) 'Model parameters are: '
c      write(6,*) 'tr = ',tr
c define source (injection) functions
c q(i) - PAIR inject funct, s(i) - photon inject. func.
c power law(?) injection
      do 45 i = 1,nebin
       q(I) = 0.
       pq(i) = 0.
45    continue

c Take care of injection:
c ***
c *** ACHTUNG: there is a geometric factor relating le/s to
c       q,s the >dimensionless<  particle injection rates:
      geoz = 16.*atan(1.)/3.
c    (this ^ is for comparison with big Z,FGBC geoz = 1.)

      urad= 0.0
      ue = 0.0
c *factor of -1 is to agree with big Z *
      if (rle .ne. 0.) then
        if (plawin .lt. 0.) then
          write(6,*) '** Monoergetic injection at gamma =',een(igmax)
          q(igmax) = 1.
        else
c          write(6,*) '** Power law injection with index: ',plawin
c          write(6,*) '   from gmin =',een(igmin),'to gmax =',een(igmax)
          do 422 i = igmin,igmax
422         q(i) = een(i)**(-plawin)*ewidth(i)
        endif
        rntot = 0.
        do 47 i = 1,nebin
          rntot = rntot + q(i)
          ue = q(i)*(een(i)) + ue
47      continue
c remove rest mass since it is reaccelerated away
c if have pair injection then things more complicated because
c of pair annihilation -- in practice we won't bother with it
        if (pairinj) then 
c           print *,'PAIR INJECTION'
           ue = 2*ue
        else
           ue = ue - rntot
c           print *,'ELECTRON INJECTION'
        endif


        do 67 i = 1,nebin
          q(i) = q(i)/ue*rle / geoz
67      continue
        ue = 0.
        do 68 i = 1,nebin
          ue = 2.*q(i)*(een(i))*geoz + ue
68      continue
      endif
c  black body photon injection at xs
      if (rls .ne. 0.) then
        if (tr .lt. 0.) then 
c         use diskpn model
          tparam(1) = -tr*511.0
          tparam(2) = Rin
          do i=1,ngambin
            tear(i) = upwgam(i)*511.0
          enddo
          tear(0) = upwgam(1)*10.**(-0.05d0)*511.0
          call xsdiskpn(tear,221,tparam,1,tphotar)
          do i=1,ngambin
            s(i) = tphotar(i)
          enddo
c        ^^ may be a constant of 511 floating around, but gets normalized out
c           I think
        else 
c         use standard blackbody
          do 66 i = 1,ngambin
            tbb = 1.e18*gwidth(i)*wgam(i)*wgam(i)
            texp = wgam(i)/tr
            texp = .5*(texp-1.+sign(texp+1.,60.-texp))
            tbb = tbb/(exp(texp)-1.)
            s(i) = dmax1(0.d0,tbb)
66        continue
        endif
        do 48 i = 1,ngambin
48        urad = s(i)*wgam(i)+ urad
        do 49 i = 1,ngambin
          s(i) = s(i)/urad*rls/geoz
49      continue

c Normalization of disk component, will be used to renormalize total spectrum
c to make the XSPEC norm of the total spectrum equal norm of the disc spectrum.
c Thus, the inner disk radius can be computed.

        radnorm = urad * geoz / rls

        urad = 0.0
        do 50 i = 1,ngambin
50        urad = s(i)*wgam(i)*geoz+ urad
      else
        do 102 i = 1,ngambin
102       s(i) = 0.
      endif

c      write(6,*) 'urad-dot: ',urad
c      write(6,*) 'ue-dot: ',ue
c      write(6,*) 'q(61) = ',q(61)

c  initialize tables
c      if (ians1 .ne. 0)
c     &   write(6,*) 'Extended low photon energy range enabled.'
c      if (ians2 .ne. 0) then
c        write(6,*) 'Synchrotron radiation enabled.'
c        write(6,*) ' --Magnetic field strength: ',b0
c        write(6,*) ' --Source region radius: ',rad
c      endif
      if (firstcall) then 
         call load
c         write(6,*) 'file tables loaded in...?'
      endif


c  ** Initial guesses for various parameters
       do 766 i = 1,nebin
766      edist(i) = 0.

       edist(1) = taup  
       posdist(1) = 0.d0
        
        do i=1,ngambin
          gdist(i) = s(i)
        enddo

c      write(6,*) 'o.k. here goes nothing...'
      return
      end

      subroutine thcsolve
      include "common.eq"
      dimension pstore(241)

c -- set up constants
      dlg = log(een(5))-log(een(4))
      do 876 i=2,nebin
876     deltag(i) = een(i)-een(i-1)
      tenesc = (rle+rls+rlth)/geoz

      opy = 0.
      itherm = 1
      nphot = 0

      u = een(1)
      a  = 45./4.
      b = (27.-15.*u)/8.
      c= 1. - u

c iteration difference to stop at before start thermal
      tdiff1 = 1.d0
c iterate until escaping energy has converged to better than econv
      econv = 2d-5

c  --- start main loop
c  --- (assume theta, eth, edist(1) initialized in setup! Ja?)
c  aber nicht otheta
      otheta = theta
      oetotal = 0

      niter = 0
      do i=1,ngambin
        otesc(i) = 1.
        pstore(i) = 0.
      enddo
      psum = 0

1     continue
      niter = niter + 1

      do 20 i = 1,ngambin
        esc(i) = 0.
	pout(i) = 0.
	pin(i) = s(i)
20    continue

c   -- now do photon distribution
      call thcdopcomp

      do i=1,ngambin
        otesc(i) = tesc(i)
      enddo



      enesc = 0.0
      do i=1,ngambin
        enesc = enesc + esc(i)*wgam(i)
      enddo
c      print *,'energy escaping',enesc,'vs. expected',tenesc

      
      end 



c ----------------------------------------

       subroutine thcdotherm
       include "common.eq"


c -- START: check temperature first
c      if (theta .gt. .15) write(6,*) 'Warning theta too high!',theta
c      if (theta .lt. .001) write(6,*) 'Warning theta too low!',theta

      x = log10(theta)
c*********      IDIOTSKI DON'T FORGET
      eth=(1.d0+27.d0/8.d0*theta+45.d0/8.d0*theta*theta)
     &   /(1.d0+15.d0/8.d0*theta)*(edist(1)+posdist(1))


c ** initialize escape array
c ** Remember N for Tau_thomp = (ne- + ne+)=2!!!*edist(1)
         tau = 1./3.*(edist(1)+posdist(1))
         do 303 i = 1,ioff
           tesc(i) = 1.d0/(1.d0+tau)
303      continue
930    do 30 i = 1+ioff,81+ioff
         ii = i-ioff
         if (theta .lt. .15) then
           trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
         elseif ((theta .ge. .15).and.(theta .lt. .16)) then
           x1 = log10(.15)
           x2 = log10(.16)
           trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
           qrate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
           qrate=10.**qrate
           trate = trate+(qrate-trate)/(x2-x1)*(x-x1)
           trate=dmin1(1.d0,trate)
         elseif (theta .ge. .16d0) then
           trate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
           trate=10.**trate
           trate=dmin1(trate,1.d0)
         endif
         tau = 1./3.*trate*(edist(1)+posdist(1))
         tesc(i) = 1./(1.+tau)
30     continue
       do 309 i =82+ioff,100+ioff
         ii = i - ioff
         if (theta .lt. .15) then
           trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
         elseif ((theta .ge. .15).and.(theta .lt. .16)) then
           x1 = log10(.15)
           x2 = log10(.16)
           trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
           qrate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
           qrate=10.**qrate
           trate = trate+(qrate-trate)/(x2-x1)*(x-x1)
           trate=dmin1(1.d0,trate)
         elseif (theta .ge. .16) then
           trate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
           trate=10.**trate
           trate=dmin1(trate,1.d0)
         endif
         tau = 1./2.7*trate*(edist(1)+posdist(1))*(1.-wgam(i))
         tesc(i) = 1./(1.+tau)
309    continue
       do 399 i =101+ioff,181+ioff
         tesc(i)= 1.d0
399    continue
101    return
       end












