c    written by Paolo  Coppi 
c    changed by Juri to increase the speed (Oct 28, 1997) 
c    relativistic reflection is added Oct 29, 1997
c    parameters changed by MG on Nov 5, 1997
c    diskbb replaced by diskpn by MG, Jun 28, 1998
c    fits data load routines added by PC, Jul 23, 2001

      SUBROUTINE XSEQPAIR(Ear,Ne,Param,Ifl,Photar,Photer)
      include "common.eq"
      common /normdisk/ radnorm

      INTEGER Ne , Ifl
      REAL*4 Param(20) , Ear(0:Ne) , Photar(Ne), Photer(Ne)

c
c     driver for the nonthermal/thermal pair program
c
c     number of model parameters: 18
c     parameters are given below in the order of decreasing importance:
c     1: hard-to-soft compactness ratio
c     2: blackbody compactness (if <0, uncomptonized black body component only)
c     3: blackbody temperature in eV (if <0, use diskpn, T_{max}=abs(Par(3)) )
c     4: non-thermal-to-hard compactness ratio (0 for pure thermal plasma)
c     5: Thomson optical depth of ionization electrons (e.g., 0)
c     6: source region radius in cm (for Coulomb/bremsstrahlung only)
c     7: minimum Lorentz factor for power law injection (not used for mono)
c     8: maximum Lorentz factor for power law injection 
c     9: power law injection index (if <0, inject monoenergetically at #8)
c     10 choose primary pair or electron injection (0=electron, 1=pair)
c     11: cosIncl, cosine of inclination angle
c     12, R, amount of reflection (if < 0 then only reflection component)
c     13: FeAb, iron abundance in units of solar
c     14: abundance of elements heavier than He relative to solar
c     15: disk temperature (used for reflection)
c     16: xi, ionization parameter
c     17: beta, reflection emissivity law (r^beta) IF=-10 then 
c         non-rotating disk, IF=10 then  (1.-sqrt(6./rg))/rg**3
c     18: radin, inner radius of the disk in M
c     19: radout, outer radius of the disk in M
c     20: redshift
c
c    Note: if #3 OR #12 < 0, only show BB and/or Reflected components
c
c     algorithm:
c           a(x)=(non-thermal/thermal pair spectrum)+reflection
c

      INTEGER n , np ,  ierr, nem,ne1,neold 
      parameter (nem=5000)
      parameter (np=20,nref=9,nrefl=7,ndisk=2)
      REAL*4 pa0(np),parref(nref),par_refl(nrefl),RedShift,RefScale 
      REAL*4 par_disk(ndisk),photdisk(nem)
      REAL*4 p1, p4
      real*8 xg(20),wg(20),wkev(221),
     & phref(nem),photco(nem),photre(nem),obj(nem),objold(nem),   
     & phbb(nem),photbb(nem)
      common/compsp1/ ospect(221)

      real*4 tear(0:221)
 
      LOGICAL firstcall , fl0 , fl1 , fl2 
      CHARACTER contxt*127

      SAVE firstcall, fl0, fl1, fl2, pa0, parref
      SAVE xg,wg,neold,objold,RedShift,RefScale
      SAVE phref,photre,photco,phbb,photbb

      DATA firstcall/.TRUE./,fl0/.true./,fl1/.true./,fl2/.true./
      DATA pa0/np*999./,parref/nref*999./,RedShift/0./
      DATA neold/999/ 
      logical showbb


c change input XSPEC parameters:
c
c   p(1) = (l_th + l_nth) / l_bb
c   p(2) = l_bb
c   p(4) = l_nth / (l_th + l_nth)
c
c into new values:
c
c   p(1) = l_nth
c   p(2) = l_bb
c   p(4) = l_th

      if (Param(2) .lt. 0.0) then
        Param(2) = -Param(2) 
        showbb  = .true.
      else
        showbb = .false.
      endif

      p1 = Param(1) * Param(2) * Param(4)
      p4 = Param(1) * Param(2) * (1.0 - Param(4))
      Param(1) = p1
      Param(4) = p4

      print *, 'l_nth = ', p1
      print *, 'l_th  = ', p4 

c object's frame energies (in mc2)
      ne1=ne+1
         do i=0,ne
            obj(i+1)=dble((1.+Param(20))*ear(i)/511.)
         enddo

      IF ( firstcall ) THEN
         CALL xwrite('     ',5)
         CALL xwrite('            Eqpair V1.01, FITS data version',5)
         CALL xwrite('Still experimental  -- use at your own risk!',5)
         CALL xwrite('For questions, contact Paolo Coppi',5)
         CALL xwrite('at coppi@astro.yale.edu .',5)

         call gauleg(0.d0,1.d0,xg,wg,10)
      ENDIF


c  calculate internal source spectrum if necessary
cjp checking only for the first 10 parameters (forget about accuracy)
      DO 100 n = 1 , 10 
         IF ( Param(n).NE.pa0(n) ) fl0 = .TRUE.
 100  CONTINUE
c Check also R_in
      if (Param(18) .ne. pa0(18)) fl0 = .TRUE.
c      if (fl0) print *,'need to recompute model'
      IF (fl0.or.Param(20).ne.RedShift) THEN
         CALL pairiter(Param,firstcall,np)
          do i=1,221
c total emergent spectrum including black body
              esc(i) = esc(i)/gwidth(i)
c separate Comptonized continuum and bb!!!! 
c
              bbspect(i) = bbspect(i)/gwidth(i)
              ospect(i)=esc(i)-bbspect(i)
c ACHTUNG
c    can't allow zeros or things get all messed up 
c    (spect interp. takes log(0)-- oops)
              ospect(i) = max(1.d-30,ospect(i))
              bbspect(i) = max(1.d-30,bbspect(i))
              tspect(i) = max(1.d-30,tspect(i))
          enddo
c screws up near annihilation line!
c          call qspline(wkev,ospect,221,1d30,1d30,f2)

c renormalize to disk model
          fnorm = radnorm / 511.0

c      xn = (1.d0+Param(20))/511. 
c interpolate to get spectrum at 1 KeV
c        targ = xn
c        i = max(min(int(log10(targ)*20.d0+141.5d0),221),1)
c        if (targ.lt.wgam(i))  i=max(i-1,1)
c        i2 = min(221,i+1)
c        x0 = wgam(i)
c        x1 = wgam(i2)
c        y1 = ospect(i2)
c        y0 = ospect(i)
c        if (i.ne.i2) then 
c           tempeqp = y0*exp(log(y1/y0)/log(x1/x0)*log(targ/x0))
c        else
c           tempeqp = y0
c        endif
cjp normalizing only continuum radiation without reflection 
c        fnorm = 1.d0/tempeqp

          do i=1,221
            ospect(i) = ospect(i)*fnorm
          enddo

      ENDIF
 


cc ** Now add reflection!
c      
c check obj 
      if(neold.ne.ne) fl2=.true.
      do i=1,ne1 
      if(obj(i).ne.objold(i)) fl2=.true.
      enddo 
c check if reflection parameters have changed 
      do i=13,19 
      if(param(i).ne.parref(i-10)) then 
      fl1=.true. 
      endif 
      enddo 
      if(param(11).ne.parref(1)) fl1=.true.
      if(RedShift.ne.Param(20)) fl1=.true.

c reflection 
c normalization 
      RefScale=abs(Param(12)) 
      IF((fl0.or.fl1.or.fl2).and.(RefScale.gt.1e-6.or.firstcall)) THEN 
cc ionized reflector (and possibly rotating) 
c  ab0, iron log10 abundance relative to solar
      par_refl(1)=alog10(Param(13)) 
c  ab_, log10 abundance of elements heavier than He relative to
c       the solar abundances
      par_refl(2)=alog10(Param(14))
c  xi, ionization parameter
      par_refl(3)=Param(16)
c  temp, disk temperature (for ionization balance, Tmax=10^6 K)
      par_refl(4)=Param(15)
c  beta      - reflection emissivity law (r^beta)
      par_refl(5)=Param(17)
c  radin     - inner radius of the disk
      par_refl(6)=Param(18)
c  radout    - outer radius of the disk
      par_refl(7)=Param(19)
c cos angle 
      angle = Param(11)
c     wgam - energies (221)
c   ospect  - angle averaged comptonized spectrum 
cjp ACHTUNG: here one should have ONLY comptonized spectrum
cjp NOT bb+comptonized !!!!!!!! 
c     obj - energies where refl should be computed
c     phref  - reflected spectrum at a given angle
      irefl=0
c relativistic disk reflector
      if(Param(17).ne.-10.) irefl=1 
      call comprefl1(wgam,ospect,obj,phref,angle,
     + par_refl,nrefl,221,ne1,irefl)
      ENDIF 

c    rebinning of the comptonized spectrum
      IF(fl0.or.fl2.or.Param(20).ne.RedShift) THEN 
         do i=1,221
           wkev(i) = wgam(i)*511d0
         enddo
      do it=0,Ne-1
        xlo = Ear(it)
        xhi = Ear(it+1)
        dx = xhi-xlo
        sum = 0.d0
        do j=1,10
          targ = (xg(j)*dx + xlo)*(1.d0+Param(20))
c--- inline expansion
          if ((targ.lt.wkev(1)).or.(targ.gt.wkev(221))) then 
            tempeqp = 1d-30
          else
            i = max(min(int(log10(targ/511.d0)*20.d0+141.5d0),221),1)
            if (targ.lt.wkev(i))  i=max(i-1,1)
            i2 = min(221,i+1)
            x0 = wkev(i)
            x1 = wkev(i2)
            y1 = ospect(i2)
            y0 = ospect(i)
            if (i.ne.i2) then 
              tempeqp= y0*exp(log(y1/y0)/log(x1/x0)*log(targ/x0))
            else
              tempeqp = y0
            endif
          endif
          sum = sum + tempeqp*wg(j)*dx
        enddo
        photco(it+1) = sum
      enddo

c    rebinning of the bb spectrum
      do it=0,Ne-1
        xlo = Ear(it)
        xhi = Ear(it+1)
        dx = xhi-xlo
        sum = 0.d0
        do j=1,10
          targ = (xg(j)*dx + xlo)*(1.d0+Param(20))
c--- inline expansion
          if ((targ.lt.wkev(1)).or.(targ.gt.wkev(221))) then 
            tempeqp = 1d-30
          else
            i = max(min(int(log10(targ/511.d0)*20.d0+141.5d0),221),1)
            if (targ.lt.wkev(i))  i=max(i-1,1)
            i2 = min(221,i+1)
            x0 = wkev(i)
            x1 = wkev(i2)
            y1 = bbspect(i2)
            y0 = bbspect(i)
            if (i.ne.i2) then 
              tempbb= y0*exp(log(y1/y0)/log(x1/x0)*log(targ/x0))
            else
              tempbb = y0
            endif
          endif
          sum = sum + tempbb*wg(j)*dx
        enddo
c ** DON'T FORGET normalization
        sum = sum * fnorm
        photbb(it+1) = sum
      enddo
      ENDIF 

c -    now get reflection component
      IF(RefScale.gt.1e-6) THEN
       IF(fl2.or.fl1.or.fl0.or.Param(12).ne.parref(2)) THEN 
c integrate over the bin size
       do i=1,Ne
       photre(i)=(phref(i+1)+phref(i))*dble((ear(i)-ear(i-1))/2.)
     &  *RefScale
       enddo
       ENDIF 
      ELSE 
      do i=1,Ne
      photre(i)=0. 
      enddo
      ENDIF 


      do i=1,Ne
        photar(i) = 0.0
        if ((Param(12).lt.0.).or.(showbb)) then
          if (Param(12).lt.0.) photar(i)=photre(i)+photar(i)
          if (showbb)          photar(i)=photbb(i)+photar(i)
        else
           photar(i)=sngl(photco(i)+photre(i)+photbb(i))
        endif
      enddo

         Do n = 1 , np
            pa0(n) = Param(n)
         EndDo
         fl0 = .FALSE.

         Do n = 11 , 19
            parref(n-10) = Param(n)
         EndDo
         RedShift=Param(20) 
         fl1 = .FALSE.

         Do i = 1, Ne1 
            objold(i)=obj(i)
         EndDo
         neold=ne 
         fl2 = .FALSE.

         firstcall = .FALSE.
c      print *,'o.k..finished w/eqpair'

      return
 
 
 999  CONTINUE
      IF ( ierr .NE. 0 ) THEN
         CALL xwrite(contxt, 10)
      ENDIF
 
      RETURN
      END

c ===================================


      subroutine pairiter(param,firstcall,np)

      include "common.eq"

      real*4 param(np)
      logical firstcall


c set model parameters and initialize model
      call setup(param,firstcall,np)

c double check for bad input parameters
      if((rle .lt. 1.d-40).and.(rlth .lt. 1.d-40)
     &   .and.(taup.lt.1d-40)) then
c        do nothing!!  (no pairs!, gdist(i) = s(i) in setup)
         do i=1,ngambin
            esc(i) = gdist(i)
         enddo
         return
      endif
      if((rle .lt. 1.d-40).and.(rlth .gt. 1.d-40)
     &   .and.(taup.lt.1d-40)) then
         print *,'ILLEGAL INPUT COMBINATION:  l_th is not zero, yet you'
         print *,'have no background electrons/positrons since tau_p=0'
         print *,'and l_nth=0!!     '
         pause
         do i=1,ngambin
            esc(i) = gdist(i)
         enddo
         return
      endif


c get model spectrum
      call solve

c get uncomptonized blackbody spectrum

      call bbesc

      return
      end

c ---------------------------
      subroutine dopcomp
      include "common.eq" 
      real*8 ntscat(221,221)
      data thresh /0.1/

      if ((diff .lt. 0.3d0).and.(niter.gt.5)) then 
         thresh = 0.1d0
      else
         thresh = 2.d0
      endif
c      print *,'thresh',thresh,diff

       iphot = 0

c assume pout cleared, pin set to s(i)
c assume e- dist fixed

      do i=2,nebin
        tau = edist(i) + posdist(i)
        do j=1,ngambin
          if (compr(i,j).gt.0) then
            pout(j) = pout(j) + tau*compr(i,j)
          endif
        enddo
      enddo


c call doesc + annihilation + brem + anything else that makes photons
       do i=1,ngambin
         gder(i) = 0.d0
       enddo
       call dotherm
       call dobrem


     

       do i=1,ngambin
         pout(i) = pout(i) + tesc(i)
         pin(i) = pin(i) + gder(i)
         pin0(i) = pin(i)
       enddo

c get nonthermal scattering matrix
c throw out row one
c can optimize this out if you want later
        do i=1,221
          do j=1,221
            ntscat(i,j) = 0.
          enddo
        enddo

        index = 0
        i=1
        do j=1,221
          do k=ictab(i,j,1),ictab(i,j,2)
            index=index+1
          enddo
        enddo
 
       do i=2,81
         tau = posdist(i) + edist(i)
         do j=1,221
           rate = tau
           do k=ictab(i,j,1),ictab(i,j,2)
             index=index+1
             ntscat(k,j) =ntscat(k,j)+ctab(index)*rate
           enddo
         enddo
       enddo

       do i=1,ngambin
         pout(i) = pout(i) - ntscat(i,i)
         ntscat(i,i) = 0
       enddo



       if (itherm.eq.1) then
         call mkscat
       endif


      do i=1,ngambin
         pout0(i) = pout(i)
      enddo

c  now iterate


1     continue

      psum = 0.
      do 10 i = 141,221
      do 20 j = 1,i
        ii = i-140
        if(ii.gt.221) pause 'hi'
        if(ii.lt.1) pause' lo'
        if (j.gt.221) pause 'hj'
        if(j.lt.1) pause 'lj'
        if (ppr(ii,j).lt. .005) goto 20
        if (ipp(ii,j) .lt. 1) goto 20
        erate = gdist(i)*gdist(j)*ppr(ii,j)
        psum = psum + erate
        pout(i) = pout(i) + ppr(ii,j)*gdist(j)
        pout(j) = pout(j) + ppr(ii,j)*gdist(i)
20    continue
10    continue


      do i=1,221
        do j=1,221
          pin(i) = pin(i) + gdist(j)*ntscat(i,j)
        enddo
      enddo


      if (itherm.eq.1) then
        do i=1,ngambin-20
           do j=iscat(i,1),iscat(i,2)
              pin(j) = pin(j) + scat(i,j)*gdist(i)
           enddo
        enddo
      endif


      dmax = 0
      do i=1,ngambin
        temp = pin(i)/pout(i)
        odmax = dmax
        dmax = max(abs(temp-gdist(i))/(gdist(i)+1.d-40),dmax)
        if (dmax.ne.odmax) idmax = i
        gdist(i) = temp
      enddo


      if (dmax .gt. thresh) Then
         do i=1,ngambin
           pout(i) = pout0(i)
           pin(i) = pin0(i)
         enddo
         goto 1
      endif


      do i=1,ngambin
        esc(i) = tesc(i)*gdist(i)
      enddo


      return
      end

c =========================================================
c dump out particle distributions

      subroutine info
      include "common.eq"

c      backspace(1)
      nz = nz + 1
      write(1,*)'------------------------------------------------'
      write(1,*)'''ITERATION# ',iter,' le =',rle,' ls =',rls,''''
      write(1,*) '''rlth =',rlth,'taup=',taup,''''
      write(1,*)'''-- iterations since last report: ',iter-noiter,''''
      write(1,*) '''Locomp , sync '''
      write(1,*) ians1,ians2
      write(1,*) '''B0,  Source Rad(m.) '''
      write(1,*) b0,rad
      noiter = iter
      write(1,*)'''Time = '''
      write(1,*) time
      etot = 0.
      trnpart = 0.
      do 30 i = 2,nebin
       etot = etot + een(i)*(edist(i)+posdist(i))
       trnpart = trnpart+(edist(i)+posdist(i))
30    continue
      etot = etot+ 2.*eth
      trnpart = trnpart + (edist(1)+posdist(1))
      enesc = 0.
      do 31 i = 1,ngambin
       etot = etot + wgam(i)*gdist(i)
       trnpart = trnpart+gdist(i)
       enesc = geoz * esc(i)*wgam(i) + enesc
c       print *,i,esc(i),gdist(i)
31    continue
      write(1,*) '''theta = '''
      write(1,*) theta,xs
      xdiff = abs(trnpart - rnpart)
      rnpart = trnpart
      write(6,*) 'Etotal = ',etot
      write(6,*) 'rnpart = ',rnpart
      write(6,*) 'idump = ',idump
      write(6,*) ' dt = ',dt
      write(1,*)'''Etotal = ''',etot
      write(1,*)'''Total # of particles:''',rnpart
      write(1,*)
      write(1,*) '''rate cascading into bin 2: ''',deltan
      write(1,*) '''minimum timestep in e- bin ''',idtmin,' = ',dt
      write(1,*)'''e- bin values: (Gam,min,max) ''',plawin,igmin,igmax
      do 10 i = 1,nebin
c      write(1,*)een(i),edist(i),eder(i),gnder(i),gcder(i)
      write(1,299)een(i),edist(i),posdist(i),gcder(i)
c299   format(2x,g12.6,4x,g13.7,4x,g13.7)
299   format(2x,g12.6,4x,g13.7,4x,g13.7,4x,g13.7)
10    continue
      write(1,*)
      write(1,*)'''photon bin values:'''
      sum = 0.
      do 20 i = 1,ngambin
        sum = sum + wgam(i)*esc(i)*geoz
20      write(1,399) wgam(i),geoz*s(i),geoz*esc(i),sum
399     format(2x,g12.6,4x,g13.7,4x,g13.7,4x,g13.7)
      write (1,*) '''Escaping photon luminosity: ''',enesc
      write (1,*) '''Escaping pair   luminosity: ''',.3*eth*2.*geoz
      print *,nz
      write (1,*) 'Pair yield: ',py ,nz
c -- For Piero
c      write(1,*) '''Tau_comp(x): '''
c      do 88 i = 1,ngambin
c        tau = 0.
c        do 89 j = 2,nebin
c          tau = tau+2.*ecomp(j,i,1)*edist(j)
c89      continue
c        write(1,499) wgam(i),tau
c499     format(2x,g12.6,4x,g13.7)
c88    continue
c      endfile(1)
      call flush(1)
c	do i=1,ngambin
c          gder(i) = 0.
c	enddo
c	call dotherm
c	
c	do i=1,ngambin
c	write(2,*) wgam(i),geoz*gder(i)*tesc(i)
c        enddo
90    return
      end

c============================================

      subroutine getth(iread)
      include "common.eq" 
      external dtime
      real dtime
      real time(2)
      logical iboink

c  ------   Get Optical Depth in equilibrium
c for pair inject
      if (pairinj) then
        ein(1) = psum + erninj
        posin(1) = psum + erninj
      else
        ein(1) = psum
        posin(1) = psum
      endif

c new experiment -- let's compute tau dynamically again and see what 
c happens
c      if (iread.gt.0) then
        call getethd
        sarg = sqrt(taup*taup+4.d0*ein(1)/eout2(1))/2.d0
        edist(1) = sarg+0.5d0*taup
        posdist(1) = sarg-0.5d0*taup
c        print *,'NEW TAU : ',posdist(1) + edist(1),n
      
 
        call dotherm
   
        do i=1,ngambin
          gdist(i) = gdist(i)*otesc(i)/tesc(i)
c muy important 
          otesc(i)  = tesc(i)
        enddo
c      endif


      realotheta = theta

 888  continue
      if (itherm.gt.0) then

         ith= 0
         dnth = 2.d0*psum
c pair injection
         if (pairinj) then 
            ethder0 = dnth*een(1) + direthder + 2.d0*erninj*een(1)
         else
c electron injection
            ethder0 = dnth*een(1) + direthder
     &                  +(een(1)-1.d0)*(erninj+prninj)
         endif

c try a trial temperature and figure out which the gradient
c of ethder is pointing
           temp = theta
           if (ethder .gt. 0.) then
             theta = theta*1.001
           else
             theta = theta*.999
           endif
           ethder = ethder0
           call getethd
c ** don't forget coulomb heating & brems
           ctemp = ethder
	   call docoul
           coulethd = ethder-ctemp
           call dobrem
           oethder = ethder
           otheta = theta
           theta = temp
           otau = edist(1)
           yeoldth = otheta
           iboink = .false.
           dn = 1.0 
336     continue
           ethder = ethder0
           call getethd
c ** don't forget coulomb heating & brems
c	   call docoul
c BUT don't recompute coul heating during temperature search
c too unstable!
           ethder=ethder + coulethd
           call dobrem
           if (ethder.eq.oethder) then
              print *,'temp ethder mucked!'
           endif
           if (theta.ne.otheta) drdth=(ethder-oethder)/(theta - otheta)
c           print *,'theta,otheta,ethder,oethder,drdth',theta,otheta,
c     &         ethder,oethder,drdth
           otheta = theta
           oethder = ethder
           if(drdth.ne.0) theta = theta - ethder/(drdth)*dn
           if (theta.lt.1.d-4) then
              dn = (otheta-1.d-4)/ethder*drdth
              theta = otheta
              theta = 0.5*otheta
c             print *,'theta boink!... ouch'
              iboink  = .true.
           endif
           if (theta.gt.50.d0) then
              theta = min(50.d0,1.07*otheta)
c              print *,'theta bink!...eech'
           endif
           dth = abs(theta - otheta)/(abs(theta)+1.d-21)
c           print *,'th,oth,thd,othd',theta,otheta,ethder,oethder


           ith = ith + 1
           if (ith.gt.15) then
c               print *,'ith ZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZZAP'
               if (iboink) then
                  theta = max(0.8*theta,1.d-4)
               else
                  theta = realotheta*1.2
               endif
               goto 889
           endif

           if (abs(dth) .lt. 1.d-6) goto 887
           goto 336

887        continue

889        continue

            dn = min(.5d0,1.d0/(1.d0+3.d0*realotheta))
            theta = dn*theta + (1.d0-dn)*realotheta
       endif
   
c       print *,'final theta = ',theta

      return
      end


c=====================

      subroutine load
      include "common.eq"
      CHARACTER*255 DATADIR
      integer lenact, ios, blocksize, rwmode, status
      character*255 FileName

c get free unit number from xspec
      call getlun(ilun)

      write(6,*) 'Reading data files...'

      rwmode = 0
      blocksize = 1

c  **********************************
c  Set this variable to $LMODDIR, the directory containing your local model.
c  (Use the exact definition in $LMODDIR, i.e., don't add a trailing slash.)
      DATADIR = '/usr/local/lheasoft/localmodels'
c  **********************************

      DATADIR = DATADIR(:lenact(DATADIR))//'/eqdata/'

      write(6,*) 'ltcz'
      FileName = DATADIR(:lenact(DATADIR))//'ltcz.fits'
      call ftopen(ilun, FileName, rwmode,  blocksize, status)
c clav
      call fitsread(ilun,.false.,clav,2,0,61,41)
c ig
      call ifitsread(ilun,.true.,ig,3,61,41,2)
c rldisp
      call fitsread(ilun,.true.,rldisp,3,61,41,2)
      call ftclos(ilun,status)

      write(6,*) 'comp'
      FileName = DATADIR(:lenact(DATADIR))//'comp.fits'
      call ftopen(ilun, FileName, rwmode,  blocksize, status)
c ecomp
      call fitsread(ilun,.false.,ecomp,3,61,161,3)
c gcomp
      call fitsread(ilun,.true.,gcomp,3,61,161,3)
c iecomp
      call ifitsread(ilun,.true.,iecomp,3,61,161,3)
c igcomp
      call ifitsread(ilun,.true.,igcomp,3,61,161,3)
      call ftclos(ilun,status)

      write(6,*) 'azntc'
      FileName = DATADIR(:lenact(DATADIR))//'azntc.fits'
      call ftopen(ilun, FileName, rwmode,  blocksize, status)
c av(81,221)
      call fitsread(ilun,.false.,av,2,0,81,221)
c compr(81,221)
      call fitsread(ilun,.true.,compr,2,0,81,221)
c icphot(81,221)
      call ifitsread(ilun,.true.,icphot,2,0,81,221)
c cphotf(81,221)
      call fitsread(ilun,.true.,cphotf,2,0,81,221)
c ice(81,221)
      call ifitsread(ilun,.true.,ice,2,0,81,221)
c cef(81,221)
      call fitsread(ilun,.true.,cef,2,0,81,221)
      call ftclos(ilun,status)


      write(6,*) 'clt2'
      FileName = DATADIR(:lenact(DATADIR))//'clt2.fits'
      call ftopen(ilun, FileName, rwmode,  blocksize, status)
c cltr(141,241)
      call fitsread(ilun,.false.,cltr,2,0,141,241)
c clta(141,241)
      call fitsread(ilun,.true.,clta,2,0,141,241)
c cltd(141,241)
      call fitsread(ilun,.true.,cltd,2,0,141,241)
      call ftclos(ilun,status)

      do 10 i =1,141
10      exn(i) =1.0+10.**(float(i-101)*.05)


      write(6,*) 'azcomp'
      FileName = DATADIR(:lenact(DATADIR))//'azcomp.fits'
      call ftopen(ilun, FileName, rwmode,  blocksize, status)
c compr(81,221)
      call fitsread(ilun,.false.,compr,2,0,81,221)
c compav(81,221)
      call fitsread(ilun,.true.,compav,2,0,81,221)
c ictab(81,221,2)
      call ifitsread(ilun,.true.,ictab,3,81,221,2)
c indexc(81,221)
      call ifitsread(ilun,.true.,indexc,2,0,81,221)
c ctab(520483)
      call fitsread(ilun,.true.,ctab,1,0,0,520483)
      call ftclos(ilun,status)

      write(6,*) 'azecomp'
      FileName = DATADIR(:lenact(DATADIR))//'azecomp.fits'
      call ftopen(ilun, filename, rwmode,  blocksize, status)
c iectab(81,221)
      call ifitsread(ilun,.false.,iectab,3,81,221,2)
      call fitsread(ilun,.false.,clav,2,0,61,41)
c ectab(218997)
      call fitsread(ilun,.true.,ectab,1,0,0,218997)
      call ftclos(ilun,status)


      write(6,*) 'azpp'
      FileName = DATADIR(:lenact(DATADIR))//'azpp.fits'
      call ftopen(ilun, FileName, rwmode,  blocksize, status)
c ppr(81,221)
      call fitsread(ilun,.false.,ppr,2,0,81,221)
c ipp(81,221)
      call ifitsread(ilun,.true.,ipp,2,0,81,221)
c ppf(81,221)
      call fitsread(ilun,.true.,ppf,2,0,81,221)
      call ftclos(ilun,status)


      write(6,*) 'thrm'
      FileName = DATADIR(:lenact(DATADIR))//'thrm.fits'
      call ftopen(ilun, FileName, rwmode,  blocksize, status)
c thann(3,2)
      call fitsread(ilun,.false.,thann,2,0,3,2)
c binann(62,3,2)
      call fitsread(ilun,.true., binann,3,62,3,2)
c cr(162,2)
      call fitsread(ilun,.true.,cr,2,0,162,2)
c cavl(162,4)
      call fitsread(ilun,.true.,cavl,2,0,162,4)
c cavh(162,4)
      call fitsread(ilun,.true.,cavh,2,0,162,4)
c cdpl(162,4)
      call fitsread(ilun,.true.,cdpl,2,0,162,4)
c cdph(162,4)
      call fitsread(ilun,.true.,cdph,2,0,162,4)
c coul(42,3)
      call fitsread(ilun,.true.,coul,2,0,42,3)
      call ftclos(ilun,status)


      write(6,*) 'htc'
      FileName = DATADIR(:lenact(DATADIR))//'htc.fits'
      call ftopen(ilun, FileName, rwmode,  blocksize, status)
c htcr(161,4)
      call fitsread(ilun,.false.,htcr,2,0,161,4)
c htcav(161,4)
      call fitsread(ilun,.true.,htcav,2,0,161,4)
c htcd(161,4)
      call fitsread(ilun,.true.,htcd,2,0,161,4)
c htcavl(81,4)
      call fitsread(ilun,.true.,htcavl,2,0,81,4)
c htcdl(81,4)
      call fitsread(ilun,.true.,htcdl,2,0,81,4)
      call ftclos(ilun,status)

      write(6,*) 'brem'
      FileName = DATADIR(:lenact(DATADIR))//'brem.fits'
      call ftopen(ilun, FileName, rwmode,  blocksize, status)
c brem(61)
      call fitsread(ilun,.false.,brem,1,0,0,61)
c brema(7710)
      call fitsread(ilun,.true.,brema,1,0,0,7710)
c ibrem(61)
      call ifitsread(ilun,.true.,ibrem,1,0,0,61)
      call ftclos(ilun,status)

      write(6,*) 'Done.'

      call frelun(ilun)
      return
      end


      subroutine ifitsread(lun,next,dummy,naxis,nax3,nax2,nax1)
      implicit none
      integer dummy(*), bitpix, naxis, naxes(4), nelements, group,
     & firstpixel, lun , nax3,nax2,nax1, status, nullval, hdutype

      logical next, anyf

        if (next) call ftmrhd(lun, 1, hdutype, status)

	naxes(3) = nax3
        naxes(2) = nax2
        naxes(1) = nax1
        bitpix = 32
        firstpixel=1
        group=1
        status = 0
        nullval = 0
        nelements = naxes(1)
        if (naxis .gt. 1) nelements=nelements*naxes(2)
        if (naxis .gt. 2) nelements=nelements*naxes(3)
c	print *,'nel i',nelements, nelements*4

	call ftgpvj(lun, group, firstpixel, nelements, nullval, 
     &    dummy, anyf, status)
        if (status .ne.0) pause 'failed ifitsread'
       return
       end

      subroutine fitsread(lun,next,dummy,naxis,nax3,nax2,nax1)
      implicit none
      integer bitpix, naxis, naxes(4), nelements, group,
     & firstpixel, lun , status, nax3,nax2,nax1, hdutype
      real*8 dummy(*), nullval
      logical next, anyf

        if (next) call ftmrhd(lun, 1, hdutype, status)

	naxes(3) = nax3
	naxes(2) = nax2
	naxes(1) = nax1
        bitpix = -64
        firstpixel=1
        group=1
        status = 0
        nullval = 0.d0
        nelements = naxes(1)
        if (naxis .gt. 1) nelements=nelements*naxes(2)
        if (naxis .gt. 2) nelements=nelements*naxes(3)
c	print *,'nel d',nelements,nelements*8

	call ftgpvd(lun, group, firstpixel, nelements, nullval, 
     &    dummy, anyf, status)

        if (status .ne.0) pause 'failed fitsread'
       return
       end



      subroutine pprod
      include "common.eq"


      psum = 0.
      do 10 i = 141,221
      do 20 j = 1,i
        ii = i-140
        if (ppr(ii,j).lt. .005) goto 20
        if (ipp(ii,j) .lt. 1) goto 20
        erate = gdist(i)*gdist(j)*ppr(ii,j)
        psum = psum + erate
c        if ((erate .lt. rmin) .or. (erate .lt. 1.e-9)) goto 99
        pout(i) = pout(i) + ppr(ii,j)*gdist(j)
        pout(j) = pout(j) + ppr(ii,j)*gdist(i)
20    continue
c       if (i.gt.181) print *,i,pout(i)
10    continue
      return
      end

c=================================
      subroutine setup(param,firstcall,np)
      include "common.eq"
      common /normdisk/ radnorm

      logical firstcall
      real*4 param(np), tparam(2),tphotar(221),tear(0:221)

c      print *,'in setup' 
      rle = param(1)
      rls = param(2)
c blackbody temperature in m_ec^2
      tr  = param(3)/5.11d5
      rlth = param(4)
      taup = param(5)
      rad  = param(6)

      Rin = param(18)

      igmax = max(min(int(log10(param(8))/.05d0+1.5d0),81),2)
      plawin = param(9)
      if (plawin .gt. 0.d0) then
        igmin = max(min(int(log10(param(7))/.05d0+1.5d0),81),2)
      endif
      if (param(10).gt.0) then
         pairinj = .true.
      else
         pairinj = .false.
      endif
      ians2 = 0
      b0 = 0.d0
      ians1 = 1
cJP is it OK? it should be param(18) but in fact it is always 0.02
      econv = param(18)
c      econv = param(16)

      maxiter = 10000

c      print *,'db: rle,rls,tr,rad',rle,rls,tr,rad
c      print *,'igmin,igmax,plawin',igmin,igmax,plawin
c      print *,'econv,maxiter',econv,maxiter

      nebin = 81
      ngambin = 221
        ioff = 40
        rioff1 = 141.5
        wmin = 10**(-7.025)
        wmax = 10.**4.025
c initial idump = bin# e- cascade into when using stationary approx
      idump = nebin
c initialize energy bin values
      do 58 i = 1,nebin
        temp = 10.**(float(i-1)*.05d0)
        een(i) = .5d0*temp*(10.**.025d0 + 10.**(-.025d0))
        upeen(i) = 10.**.025d0*temp
58    continue
      een(1) = (10.**.05d0 - 1.)/2./(10.**.025d0 - 1.)
      do 59 i = 2,nebin
59      ewidth(i) = upeen(i) - upeen(i-1)
      ewidth(1) = upeen(1) - 1.d0
c do photons
      do 62 i = 1,ngambin
        temp1 = 10.**(float(i-101-ioff)*.05d0)
        wgam(i) = .5d0*temp1*(10.**.025d0 + 10.**(-.025d0))
        upwgam(i)= 10.**.025d0*temp1
c        print *,i,wgam(i)
c        pause
62    continue
      do 60 i = 2,ngambin
        gwidth(i) = upwgam(i) - upwgam(i-1)
60    continue
      gwidth(1) = upwgam(1)-10.**(-7.025d0)
c
c  --- get model parameters
c      write(6,*) 'Hello...'
c      write(6,*) 'Model parameters are: '
c      write(6,*) 'tr = ',tr
c define source (injection) functions
c q(i) - PAIR inject funct, s(i) - photon inject. func.
c power law(?) injection
      do 45 i = 1,nebin
       q(I) = 0.
       pq(i) = 0.
45    continue

c Take care of injection:
c ***
c *** ACHTUNG: there is a geometric factor relating le/s to
c       q,s the >dimensionless<  particle injection rates:
      geoz = 16.*atan(1.)/3.
c    (this ^ is for comparison with big Z,FGBC geoz = 1.)

      urad= 0.0
      ue = 0.0
c *factor of -1 is to agree with big Z *
      if (rle .ne. 0.) then
        if (plawin .lt. 0.) then
          write(6,*) '** Monoergetic injection at gamma =',een(igmax)
          q(igmax) = 1.
        else
c          write(6,*) '** Power law injection with index: ',plawin
c          write(6,*) '   from gmin =',een(igmin),'to gmax =',een(igmax)
          do 422 i = igmin,igmax
422         q(i) = een(i)**(-plawin)*ewidth(i)
        endif
        rntot = 0.
        do 47 i = 1,nebin
          rntot = rntot + q(i)
          ue = q(i)*(een(i)) + ue
47      continue
c remove rest mass since it is reaccelerated away
c if have pair injection then things more complicated because
c of pair annihilation -- in practice we won't bother with it
        if (pairinj) then 
c           print *,'PAIR INJECTION'
           ue = 2*ue
        else
           ue = ue - rntot
c           print *,'ELECTRON INJECTION'
        endif


        do 67 i = 1,nebin
          q(i) = q(i)/ue*rle / geoz
67      continue
        ue = 0.
        do 68 i = 1,nebin
          ue = 2.*q(i)*(een(i))*geoz + ue
68      continue
      endif
c  black body photon injection at xs
      if (rls .ne. 0.) then
        if (tr .lt. 0.) then 
c         use diskpn model
          tparam(1) = -tr*511.0
          tparam(2) = Rin
          do i=1,ngambin
            tear(i) = upwgam(i)*511.0
          enddo
          tear(0) = upwgam(1)*10.**(-0.05d0)*511.0
          call xsdiskpn(tear,221,tparam,1,tphotar)
          do i=1,ngambin
            s(i) = tphotar(i)
          enddo
c        ^^ may be a constant of 511 floating around, but gets normalized out
c           I think
        else 
c         use standard blackbody
          do 66 i = 1,ngambin
            tbb = 1.e18*gwidth(i)*wgam(i)*wgam(i)
            texp = wgam(i)/tr
            texp = .5*(texp-1.+sign(texp+1.,60.-texp))
            tbb = tbb/(exp(texp)-1.)
            s(i) = dmax1(0.d0,tbb)
66        continue
        endif
        do 48 i = 1,ngambin
48        urad = s(i)*wgam(i)+ urad
        do 49 i = 1,ngambin
          s(i) = s(i)/urad*rls/geoz
49      continue

c Normalization of disk component, will be used to renormalize total spectrum
c to make the XSPEC norm of the total spectrum equal norm of the disc spectrum.
c Thus, the inner disk radius can be computed.

        radnorm = urad * geoz / rls

        urad = 0.0
        do 50 i = 1,ngambin
50        urad = s(i)*wgam(i)*geoz+ urad
      else
        do 102 i = 1,ngambin
102       s(i) = 0.
      endif

c      write(6,*) 'urad-dot: ',urad
c      write(6,*) 'ue-dot: ',ue
c      write(6,*) 'q(61) = ',q(61)

c      if don't read from file, put
c        initial non-zero starting distribution into bins
        do 87 i = 1,nebin
87        edist(i) = 1.e-7
        eth = edist(1)*een(1)
c fudge --
        theta = 2./3.*(eth/edist(1)-1.)

c  initialize tables
c      if (ians1 .ne. 0)
c     &   write(6,*) 'Extended low photon energy range enabled.'
c      if (ians2 .ne. 0) then
c        write(6,*) 'Synchrotron radiation enabled.'
c        write(6,*) ' --Magnetic field strength: ',b0
c        write(6,*) ' --Source region radius: ',rad
c      endif
      if (firstcall) then 
         call load
c         write(6,*) 'file tables loaded in...?'
      endif


c  ** Initial guesses for various parameters
       do 766 i = 1,nebin
766      edist(i) = 0.
       rinject = 0.
       do 234 i=1,nebin
234      rinject=rinject+q(i)
        
        do i=1,ngambin
          gdist(i) = s(i)
        enddo

c      write(6,*) 'o.k. here goes nothing...'
      return
      end

      subroutine solve
      include "common.eq"
      dimension pstore(241)

c -- set up constants
      dlg = log(een(5))-log(een(4))
      do 876 i=2,nebin
876     deltag(i) = een(i)-een(i-1)
      tenesc = (rle+rls+rlth)/geoz

      opy = 0.
      itherm = 0
      nphot = 0
c      theta = .6*(een(1)-1.0)
      u = een(1)
      a  = 45./4.
      b = (27.-15.*u)/8.
      c= 1. - u
      theta0 = (-b+sqrt(b*b-2.*a*c))/a

      edist(1) = 0.d0 + taup
c      print *,'>> initial guess for tau:',edist(1)

c * initial coulomb cooling rate
      cturnon = 1.d0

      erninj = 0.
      prninj = 0.
      do 877 i=2,nebin
       erninj = erninj + q(i)
       prninj = prninj + pq(i)
877   continue
      erninj = max(erninj - 1.d-20,0.d0)
c      print *,'erninj',erninj
      prninj = max(prninj -1.d-20,0.d0)
c      print *,'thermal injection compactness', rlth
      direthder = rlth/geoz


c      print *,'maximum number of iterations: '
c      read *,niter
c      print *,'starting iterations...'
      idump = 1
      otau = edist(2)

c iteration difference to stop at before start thermal
      tdiff1 = 1.d0
c iterate until escaping energy has converged to better than econv
      econv = 2d-2

c  --- start main loop
c  --- (assume theta, eth, edist(1) initialized in setup! Ja?)
c  aber nicht otheta
      otheta = theta
      oetotal = 0

      niter = 0
      do i=1,ngambin
        otesc(i) = 1.
        pstore(i) = 0.
      enddo
      psum = 0

1     continue
      niter = niter + 1

      do 20 i = 1,ngambin
        esc(i) = 0.
	pout(i) = 0.
	pin(i) = s(i)
20    continue
      ethder = 0.
      do 21 i=1,nebin
        gdot(i) = 0.
	ein(i) = q(i)
c for pair injection
        if (pairinj) then 
          posin(i) = q(i)
c for electron and/or positron injection
        else
          posin(i) = pq(i)
        endif
        eout(i) = 0.
        posout(i) = 0.
        eout2(i) =0.
        posout2(i) = 0.
21    continue
      pause = 0

c     get electron distribution
      call doelec
c   -- now do photon distribution
      call dopcomp

      do i=1,ngambin
        otesc(i) = tesc(i)
      enddo


c ---- Overall convergence check (look at photon distribution)
      diff = 0.
      do 133 i = 1,ngambin
c change for linux
        if (gdist(i).gt.1.d-38) temp=abs(pstore(i)-gdist(i))/gdist(i)
        if (temp.gt.diff) then 
           id = i
           diff = temp
        endif
133   continue

c don't forget to update pstore
      do 132 i =1,ngambin
 132     pstore(i) = gdist(i)

c      print *,'DIFF = ',diff,theta,edist(1),id
c      print *,'tau = ',edist(1)+posdist(1),'py = ',psum
c      print *,'niter',niter

      enesc = 0.0
      do i=1,ngambin
        enesc = enesc + esc(i)*wgam(i)
      enddo
c      print *,'energy escaping',enesc,'vs. expected',tenesc

      if ((diff .lt. tdiff1).or.
     &    (nphot .gt. maxiter)) then
c         print *,'o.k. passed diff test'
        if (nphot .gt. maxiter) then 
           print *,' >> Run ended prematurely: max iter hit'
           return
        endif
       ediff = abs(enesc-tenesc)/(tenesc+1.d-12)
       if((ediff .lt. econv).and.(itherm.gt.0)) then
c          print *,'convoiged!!'
c          print *,'*** theta = ',theta*511.d0,' kev',
c     &            ' tau_T = ',edist(1)+posdist(1)
c          open(unit=1,file='final',status='unknown',form='formatted')
c          call info
c          close(1)
          return
        endif
        if (itherm .eq. 0) then
          itherm= 1
          tdiff1=1.d-2
          goto 1
        endif
      endif
      
c -- go back for another round of convergence talks
      goto 1     

      end 



c ================================
      subroutine dobrem
      include "common.eq"

c *** calculate brems spectra. Careful with factors of two.!
c (ethder is TOTAL energy now)
c     common /bremd/ spec(241)
c coeffs for Stepney fit:
      dimension ba(13),bb(13),bc(13),bd(13),ga(13),gb(13),gc(13)
      dimension  gd(13),fiten(13)
      dimension store(241),tg(81)
      data ba / 1.584,1.357,1.197,1.023,
     *          .883, .700 , .572,.484,
     *          .417, .361, .322 , .286,
     *          .259 /
      data bb / .578, .437,.291,.204,
     *          .0835,-0.0494,-.139,-.181,
     *          -.209,-.240, -.244,-.257,
     *          -.258 /
      data bc / 4.565,3.842,3.506,3.036,
     *          2.831,2.545,2.352,2.175,
     *          2.028,1.914,1.795,1.705,
     *          1.617 /
      data bd / 2.091,1.855,1.672,1.593,
     *          1.487,1.364,1.254,1.179,
     *          1.108,1.030,0.982,0.923,
     *          0.879 /
      data ga / .0387,.0633,.0862,.128,
     *          .159, .208, .234, .245,
     *          .248, .247, .243, .239,
     *          .235 /
      data gb / .523,.540,.569,.596,
     *          .658,.633,.643,.695,
     *          .729,.756,.763,.755,
     *          .735 /
      data gc / 5.319,4.412,3.897,3.383,
     *          2.974,2.738,2.424,2.025,
     *          1.716,1.457,1.271,1.140,
     *          1.060 /
      data gd / .782,.689,.633,.523,
     *          .532,.326,.302,.394,
     *          .453,.500,.515,.508,
     *          .478 /
      data fiten / 0.0978,.1468,.1957,.2935,
     *             .39139,.5871,.7828,.9785,
     *             1.1742,1.3699,1.5656,1.7613,
     *             1.956947 /
c  coeffs for Haug fit:
      dimension ca(11),cb(11),cc(11),cd(11),ce(11),ha(11),hb(11),hc(11)
      dimension  hd(11),he(11),bfiten(11)
      data ca / 3808.,657.,420.3,130.0,67.55,
     *          40.72, 20.28,12.91,10.52,9.61,9.22 /
      data cb / -2560.,-4427.,250.9,-0.8,-18.79,
     *          -28.02, -29.46,-22.30,-12.14,-3.04,4.52 /
      data cc / 1036.,-114.,120.8,28.1,12.61,
     *          3.26,-1.27, -3.56, -2.97, -2.52, -2.16 /
      data cd / 7237.,7209.,0.,134.0,111.33,
     *          107.21, 91.86, 75.21, 56.00, 39.96, 27.09 /
      data ce / -539.,-4927.,566.9,91.9,36.47,
     *          8.59,-4.59,-1.88,8.54,18.28,26.31 /
      data ha / 17.619,5.124,2.633,2.854,3.315,
     *          3.629,3.991,4.262,4.366,4.383,4.374 /
      data hb / -388.0,-98.27,-10.22,8.583,11.86,
     *          13.33, 14.56, 13.26, 12.56, 12.07, 11.29 /
      data hc / 4015.7,1132.4,242.9,99.08,76.26,
     *          63.96, 47.92, 44.57, 39.36, 34.58, 32.34 /
      data hd / 3587.6,1649.4,454.4,155.6,70.38,
     *          42.07, 26.37, 12.06, 9.05, 8.59, 5.86 /
      data he / -535.0,-406.0,-122.8,-40.24,-14.01,
     *          -7.19, -5.57, -0.82,-0.80,-1.38,-0.46 /
      data bfiten / 0.01957,.03913,0.0978,.1957,.2935,
     *              .39139,.5871,.7828,.9875,1.1742,1.3699 /

      otheta = theta
      theta = max(1.d-4,theta)

      if (itherm.gt.0) then 

      if (theta .le. bfiten(11)) then
c  use Haug/Stepney approx

c take care of  thermal bremsstrahlung.
c  Use Stepney for e-e-/e+e+  - typically accurate to w/in 10% over all range.
      if (theta .le. fiten(1)) then
c   use fudged up Haug 1975b non-rel approx; not so great, but o.k.
        iswitch = min(int(log10(theta*1.05)/.05+rioff1),ngambin)
        imax = min(int(log10(theta*10.)/.05+rioff1),ngambin)
        coeff = (edist(1)*edist(1)+posdist(1)*posdist(1))
     &      /137.*sqrt(fiten(1))/sqrt(theta)
        do 161 i = 1,iswitch
          x = wgam(i)/theta
          rndot=
     &gwidth(i)*coeff/x*exp(-x)*((-ba(1)-bb(1)*x)*log(x)+bc(1)+bd(1)*x)
          gder(i) = gder(i) + rndot
          ethder = ethder - wgam(i)*rndot
161      continue
        do 171 i = iswitch+1,imax
          x = wgam(i)/theta
          rndot =
     &    gwidth(i)*coeff/x*exp(-x)*(ga(1)*x*x+gb(1)*x+gc(1)+gd(1)/x)
          gder(i) = gder(i) + rndot
          ethder = ethder - wgam(i)*rndot
171      continue
      else
        ttheta = theta
        if (theta .gt. fiten(13)) then
          write(6,*) 'TH EE BREM: THETA TOO HIGH!!'
          ttheta = fiten(13)
        endif
        it = 1
c bozito!! the energies aren't evenly spaced!!
        do 96 i = 1,13
96        if (ttheta .ge. fiten(i)) it = i
        it = min(it,12)
        delta = (ttheta-fiten(it))/(fiten(it+1)-fiten(it))
        rba = ba(it) + (ba(it+1)-ba(it))*delta
        rbb = bb(it) + (bb(it+1)-bb(it))*delta
        rbc = bc(it) + (bc(it+1)-bc(it))*delta
        rbd = bd(it) + (bd(it+1)-bd(it))*delta
        rga = ga(it) + (ga(it+1)-ga(it))*delta
        rgb = gb(it) + (gb(it+1)-gb(it))*delta
        rgc = gc(it) + (gc(it+1)-gc(it))*delta
        rgd = gd(it) + (gd(it+1)-gd(it))*delta
        imin = max(int(log10(theta*.05)/.05+rioff1),0)
        temp = gder(imin)
        iswitch = min(int(log10(ttheta*1.05)/.05+rioff1),ngambin)
        imax = min(int(log10(ttheta*10.)/.05+rioff1),ngambin)
c ***** DON'T forget the FACTOR of two -- e-e- AND e+e+ brem
c   HOWEVER, ethder is one-half total energy radiatied
c   since are only keeping track of the e- distribution.
c   (Check: energy conservation o.k.)
        coeff = (edist(1)*edist(1)+posdist(1)*posdist(1))/137.
c        do 160 i = imin+1,iswitch
        do 160 i = 1,iswitch
          x = wgam(i)/ttheta
            rndot=
     &   gwidth(i)*coeff/x*exp(-x)*((-rba-rbb*x)*log(x)+rbc+rbd*x)
c        trndot=8.*3.14/3.*exp(-x)*((rba+rbb*x)*log(1./x)+rbc+rbd*x)/x
          gder(i) = gder(i) + rndot
          ethder = ethder - wgam(i)*rndot
160      continue
        do 170 i = iswitch+1,imax
          x = wgam(i)/ttheta
          rndot = gwidth(i)*coeff/x*exp(-x)*(rga*x*x+rgb*x+rgc+rgd/x)
          gder(i) = gder(i) + rndot
          ethder = ethder - wgam(i)*rndot
170      continue
c Extend Stepney results down below .05*theta (sie werden negatif?)
c Guess low energy behavior
c        temp = gder(imin+1)-temp
c        do 172 i = 1,imin
c        rndot = temp*(1.-.28*log(wgam(i)/theta))/
c     &    (1.-.28*log(wgam(imin+1)/theta))
c         gder(i) = gder(i) + rndot
c         ethder = ethder - wgam(i)*rndot/2.0
c172   continue
      endif

c      goto 919
c929   continue
c       do 91 i = 1,ngambin
c         write(3,*) wgam(i),gder(i)-store(i)
c91       store(i) = gder(i)

c Now do e+/e- case.
c Use Haug (1987) "simple(?) analytical fits"
c Note: now there is NO factor of two in the emission coeff.
c  integral takes into account all possible combinations.
c  However, ethder must be divided by two since only half of
c  the energy radiated comes from either the pos. or e- dist.
c (but NOT in this version)
      if (theta .le. bfiten(1)) then
c   use fudge to non-rel approx. (calculating the bessel
c   function K0(x) is too costly)
        imax = min(int(log10(theta*10.)/.05+rioff1),ngambin)
        iswitch = min(int(log10(theta*1.)/.05+rioff1),ngambin)
        coeff = edist(1)*posdist(1)*2.385225686e-6/theta**1.5
        do 61 i = 1,iswitch
          x = wgam(i)/theta
          rndot= gwidth(i)*coeff/x*exp(-x)
          rndot=rndot*(cd(1)+x*ce(1)-(ca(1)+cb(1)*sqrt(x)
     *     +cc(1)*x)*log(x))
          gder(i) = gder(i) + rndot
          ethder = ethder - wgam(i)*rndot
61      continue
        do 71 i = iswitch+1,imax
          x = wgam(i)/theta
          rndot = gwidth(i)*coeff/x*exp(-x)
          rndot =rndot*(ha(1)*x*x+hb(1)*x+hc(1)+hd(1)/x+he(1)/x/x)
          gder(i) = gder(i) + rndot
          ethder = ethder - wgam(i)*rndot
71      continue
      else
        ttheta = theta
        if (theta .gt. bfiten(11)) then
          write(6,*)  'TH E+E- BREM: THETA TOO HIGH!!'
          ttheta = bfiten(11)
        endif
        do 97 i = 1,11
97        if (ttheta .ge. bfiten(i)) it = i
        it = min(it,10)
c ** NOTE: Unlike Stepney fits, here have to interpolate VALUES not Coeffs
        iu = it+1
        delta = (ttheta-bfiten(it))/(bfiten(iu)-bfiten(it))
        iswitch = min(int(log10(ttheta*1.0)/.05+rioff1),ngambin)
        imax = min(int(log10(ttheta*10.)/.05+rioff1),ngambin)
        coeff = edist(1)*posdist(1)*8.7128618e-4
        do 60 i = 1,iswitch
          x1 = wgam(i)/bfiten(it)
          x2 = wgam(i)/bfiten(iu)
          v1 = exp(-x1)/x1*(cd(it)+x1*ce(it)-(ca(it)+cb(it)*sqrt(x1)
     & +cc(it)*x1)*log(x1))
          v2 = exp(-x2)/x2*(cd(iu)+x2*ce(iu)-(ca(iu)+cb(iu)*sqrt(x2)
     & +cc(iu)*x2)*log(x2))
          rndot= gwidth(i)*coeff*(delta*(v2-v1)+v1)
          gder(i) = gder(i) + rndot
          ethder = ethder - wgam(i)*rndot
60      continue
        do 70 i = iswitch+1,imax
          x1 = wgam(i)/bfiten(it)
          x2 = wgam(i)/bfiten(iu)
          v1 = exp(-x1)/x1*((ha(it)*x1+hb(it))*x1+hc(it)+(hd(it)
     &  + he(it)/x1)/x1)
          v2 = exp(-x2)/x2*((ha(iu)*x1+hb(iu))*x2+hc(iu)+(hd(iu)
     &  + he(iu)/x2)/x2)
          rndot = gwidth(i)*coeff*(delta*(v2-v1)+v1)
          gder(i) = gder(i) + rndot
          ethder = ethder - wgam(i)*rndot
70      continue
       endif

c      do 95 i = 1,ngambin
c        write(3,*) wgam(i),(gder(i) - store(i))
c95      store(i) = gder(i)
c *** end of non-mildly rel. thermal section
       else
c ***  use  Alexanian approx, pretty good for theta > 1
c Vergessen nicht faktor von 4! (2xe-e + e+e-=2xe-e)
       coeff = 2.*.001742572369619*(0.5d0*edist(1)*edist(1)+
     &  0.5d0*posdist(1)*posdist(1) + edist(1)*posdist(1))
       eugam = .5772156649
       imax = min(int(log10(theta*10.)/.05+rioff1),ngambin)
       iswitch = min(int(log10(theta*1.05)/.05+rioff1),ngambin)
       do 92 i = 1,iswitch
         x = wgam(i) /theta
c Achtung mit dem Sign: Ei(-x) as def'd in Alex. = -!! E_1(x)  (Ab+Steg)
c ESTUPIDO !!! no es log(2theta/eugam) aber log(2theta)-eugam!!(READ next time)
         e1 = -.57721566 + x*(.99999193 + x*(-.24991055+
     &      x*(.05519968 +x*(-.00976004 + .00107857*x))))
         e1 = e1 - log(x)
         pee=28./3.+2.*x+.5*x*x+2.*(8./3.+4./3.*x+x*x)
     &     *(log(2.*theta)-eugam)+exp(x)*e1*(8./3.-4./3.*x+x*x)
         pee=coeff*exp(-x)/wgam(i)*gwidth(i)*pee
         gder(i) = gder(i) + pee
         ethder = ethder - wgam(i)*pee
92     continue
       do 93 i = iswitch+1,imax
         x = wgam(i) /theta
         e1=(x*x+2.334733*x+.250621)/(x*x+3.330657*x + 1.681534)
         e1 = e1/x/exp(x)
         pee=28./3.+2.*x+.5*x*x+2.*(8./3.+4./3.*x+x*x)
     &     *(log(2.*theta)-eugam)+exp(x)*e1*(8./3.-4./3.*x+x*x)
         pee=coeff*exp(-x)/wgam(i)*gwidth(i)*pee
         gder(i) = gder(i) + pee
         ethder = ethder - wgam(i)*pee
93     continue
       endif
c ** end thermal bremss.
c      do 98 i = 1,ngambin
c        write(3,*) wgam(i),(gder(i) - store(i))
c98      store(i) = gder(i)

        endif
c endif for itherm .gt. 0

      do i=1,ngambin
        store(i)= gder(i)
      enddo
      do i=1,nebin
        tg(i) = gdot(i)
      enddo


c finally take care of non-thermal bremsstrahlung
      index = 1
c*******************  NEBIN = 61!! careful-- should really compute
c up to nebin=81
      do 10 i = 2,61
        rnprod = edist(1)*(edist(i)+posdist(i))/4.d0
     &            + posdist(1)*(edist(i)+posdist(i))/4.d0
        eloss = brem(i)*(edist(1)+posdist(1))/2.d0
c       write(3,*)een(i),eloss
        gdot(i) = gdot(i)- eloss
        gcder(i) = gcder(i) + eloss
c       gnder(i) = gnder(i) + dmax1(eloss,0.d0)
        do 20 j = 1,ibrem(i)
          jshift  = j+ioff
          jj = j + index-1
20        gder(jshift) = gder(jshift)+rnprod*brema(jj)
        index = index + ibrem(i)
10    continue

      theta = otheta 
      return
      end

      subroutine docoul
      include "common.eq"

      if (edist(1).eq.0) return

      tethder = 0.d0

      zerox=1.0 + theta/(theta+1.0)
c I guess -- I'm putting all tau_thom in here
      rn = (edist(1)+posdist(1))/rad/6.6d-25
      wp=4*3.1415926535*4.803d-10*4.803d-10/9.1d-28*rn
c relativistic correction (approx)
      wp = sqrt(wp/(1.+3.*theta))
c      imin = max(int(log10(1.d0+4.d0*theta)/.05d0+1.5d0),2)
c to prevent bouncing around in code, need a smooth formula that
c makes gdot grow fast for  e< (1+8)theta
      echop = 1.d0+4.d0*theta
         x=(echop-1.0)/theta
         e1 = echop
         p1=sqrt(e1*e1 - 1.0)
         ps=sqrt((e1-1)/2.0)
         es=sqrt((e1+1)/2.0)
         betas=ps/es
         tmin=1.05e-27/(betas/wp)/9.1e-28/9e20/ps/es/es*(es*es+ps*ps)
         rlam=0.5-log(sqrt(2.0)*tmin)
c         print *,'rlam',rlam,log(sqrt(e1)*9.1d-28*9d20/1.05d-27/wp)
         corr= (e1-1.0)*(e1-1.0)/4.0/e1/e1*(2.0*log(2.0)+0.25)
         ap=3.0/2.0*e1/p1*(rlam+corr)
c ******************************************
c         ERROR (Moeglich) - das sollte (rlam >>+<< corr) sein
c ****************************************
c according to haug.f should be "+" so I'm changing it! PSC 8/14/97
         ap=ap/(1.0/(1.0+theta)+2.0*theta)
c    { add correction for small x}
c         if ( x < 20.0) then
         ap=ap*(x -zerox)*sqrt(x)/(x*sqrt(x)+1.0)
c  FUDGE - can go negative, because sometimes x< theta (imin approx)
c change for split e+/e- dist., Moeller and Babha scat approximately
c the same, so should be o.k.
         trechop = abs(ap*(posdist(1)+edist(1)))
      do 14 i = nebin,2,-1
         x=(een(i)-1.0)/theta
         e1 = een(i)
         p1=sqrt(e1*e1 - 1.0)
         ps=sqrt((e1-1)/2.0)
         es=sqrt((e1+1)/2.0)
         betas=ps/es
         tmin=1.05e-27/(betas/wp)/9.1e-28/9e20/ps/es/es*(es*es+ps*ps)
         rlam=0.5-log(sqrt(2.0)*tmin)
c         print *,'rlam',rlam,log(sqrt(e1)*9.1d-28*9d20/1.05d-27/wp)
         corr= (e1-1.0)*(e1-1.0)/4.0/e1/e1*(2.0*log(2.0)+0.25)
         ap=3.0/2.0*e1/p1*(rlam+corr)
c ******************************************
c         ERROR (Moeglich) - das sollte (rlam >>+<< corr) sein
c ****************************************
c according to haug.f should be "+" so I'm changing it! PSC 8/14/97
         ap=ap/(1.0/(1.0+theta)+2.0*theta)
c    { add correction for small x}
c         if ( x < 20.0) then
         ap=ap*(x -zerox)*sqrt(x)/(x*sqrt(x)+1.0)
c  FUDGE - can go negative, because sometimes x< theta (imin approx)
c change for split e+/e- dist., Moeller and Babha scat approximately
c the same, so should be o.k.
c***************  CHEcK THIS
         if (een(i).lt.echop) then 
           trate = trechop*((echop/een(i))**3.6)
         else
           trate = abs(ap*(posdist(1)+edist(1)))
         endif
c  smooth  turnon!
         trate = cturnon * trate
         gdot(i) = gdot(i) - trate
c        print *,i,een(i),ap,gdot(i),gcder(i)
c remember: TOTAL ethder
         tethder=tethder+trate*(edist(i)+posdist(i))
14    continue
      ethder = ethder + tethder
     
      return
      end


      subroutine thscat(rate,eav,disp)
      include "common.eq"

c      dimension store(241)
c      if(eav .lt.wgam(1)) then
c        write(1,*) '!tscat: eav out of bounds--',wgam(1),eav
c        eav = wgam(1)
c      endif
      tdisp = sqrt(3.)*disp
      emin = dmax1(wmin,eav-tdisp)
      emax = dmin1(wmax,eav+tdisp)
      i1 = max0(int(log10(emin)/.05 + rioff1),1)
      j1 = min0(int(log10(emax)/.05 + rioff1),ngambin)
c      do 777 i =1,ngambin
c777     store(i) = gder(i)
c  o.k. let's figure out where the peak goes    
c **** Wir haben 3 an zwei geandert.
      if (abs(i1-j1) .lt. 2) then
        it = max0(int(log10(eav)/.05 + rioff1),1)
        if (eav .eq. wgam(it)) then
          gder(it) = gder(it) + rate
        elseif (eav .lt. wgam(it)) then
          frac =(eav-wgam(it-1))/(wgam(it-1)-wgam(it))
          gder(it) = gder(it)-frac*rate
          gder(it-1)=gder(it-1)+(1.+frac)*rate
        else
          frac = (eav-wgam(it+1))/(wgam(it)-wgam(it+1))
          gder(it) = gder(it)+frac*rate
          gder(it+1)=gder(it+1)+(1.-frac)*rate
        endif
      elseif (eav .lt. wgam(i1)) then
        it = max0(int(log10(eav)/.05 + rioff1),1)
        frac =(eav-wgam(it-1))/(wgam(it-1)-wgam(it))
        gder(it) = gder(it)-frac*rate
        gder(it-1)=gder(it-1)+(1.+frac)*rate
      elseif (eav .gt. wgam(j1)) then
        it = max0(int(log10(eav)/.05 + rioff1),1)
        frac = (eav-wgam(it+1))/(wgam(it)-wgam(it+1))
        gder(it) = gder(it)+frac*rate
        gder(it+1)=gder(it+1)+(1.-frac)*rate
      else
        dw = 1./(emax - emin)
        f1 = (upwgam(i1)-emin)*dw
        f2 = (emax - upwgam(j1-1))*dw
        de2 = (upwgam(j1-1)-upwgam(i1))*dw
        de2 = de2*.5e0*(upwgam(j1-1)+upwgam(i1))
        de2 = wgam(i1)*f1+wgam(j1)*f2+de2
        if (eav .ne. de2) then
c begin
c  stick in spike to correct things. de2 is error due to
c  discrete bin energies and edge effects
        if (eav .gt. de2) then
          targ = (emax +eav)/2.
        else
          targ = (emin + eav)/2.
        endif
        it = max0(int(log10(targ)/.05 + rioff1),1)
c   ** c is fraction of # scat. part that goes into "spike"
        if (targ .eq. wgam(it)) then
          c  = (eav - de2)/(targ - de2)
          gder(it) = gder(it) + c*rate
        elseif (targ .lt. wgam(it)) then
          if ((it-1) .lt. i1) then
            c  = (eav - de2)/(wgam(i1) - de2)
            gder(i1) = gder(i1) +c*rate
          else
            c  = (eav - de2)/(targ - de2)
            frac = (targ-wgam(it-1))/(wgam(it-1)-wgam(it))
            gder(it) = gder(it)-c*frac*rate
            gder(it-1)=gder(it-1)+c*(1.+frac)*rate
          endif
        else
          if ((it+1) .gt. j1) then
            c  = (eav - de2)/(wgam(j1) - de2)
            gder(j1) = gder(j1) +c*rate
          else
            c  = (eav - de2)/(targ - de2)
            frac = (targ-wgam(it+1))/(wgam(it)-wgam(it+1))
            gder(it) = gder(it)+c*frac*rate
            gder(it+1)=gder(it+1)+c*(1.-frac)*rate
          endif
        endif
c end -- now distribute the rest of the particles uniformly
        else
          c = 0.
        endif
c (if de2 = eav, we have nothing do to of course.)
        c= (1.-c)*dw*rate
        gder(i1) =gder(i1)+(upwgam(i1)-emin)*c
        gder(j1) = gder(j1)+(emax-upwgam(j1-1))*c
        do 32 k = (i1+1),(j1-1)
32        gder(k)=gder(k) + gwidth(k)*c
      endif
c      ave = 0.
c      rpart = 0.
c      do 787 kk = 1,ngambin
c        rpart = rpart + gder(kk) - store(kk)
c787     ave = ave + (gder(kk)- store(kk))*wgam(kk)
c      if (abs(rpart -rate)/rate .gt. 1.e-5) then
c        write(6,*) 'another horrid problem!'
c        write(6,*) i,j,'supposed',rate,'actual',rpart
c        write(6,*) 'i,j: ',i1,j1,eav,de2,targ,c,frac
c        pause 'LUnch!'
c      endif
c      if (rpart .ne. 0.) then
c        ave = ave / rpart
c        if (abs(ave -eav)/ave .gt. 1.e-5) then
c          write(6,*) 'we have a big problem!!!'
c          write(6,*) i,j,i1,j1,'ave',ave,'eav',eav
c          pause 'CRUNCH!'
c        endif
c      endif
      return
      end



      subroutine getfname(stuff,n)
      implicit real*8 (a-h,o-z)
      character*8 stuff
      character*1 digit(0:9)
       stuff = 'mc'
       digit(0) = '0'
       digit(1) ='1'
       digit(2) = '2'
       digit(3) = '3'
       digit(4) = '4'
       digit(5) = '5'
       digit(6) = '6'
       digit(7)  = '7'
       digit(8) = '8'
       digit(9) = '9'
       if (n .eq. 0) then 
	 len = 1
       else
         len = int(log10(float(n))+1)
       endif
       do 10 i = 0,len-1
         id = n/10**(len-1-i) 
         id = mod(id,10)
         stuff(i+3:i+3) = digit(id)
10     continue
       stuff(3+len:3+len) = '.'
       stuff(4+len:4+len) = 'd'
       return
       end

c ----------------------------------------------
c  compute part of escaping made of black body photons that
c  do NOT scatter (presumably they are emitted from disk and
c  don't reflect off disk).

       subroutine bbesc
       include "common.eq"
       real*8 tauc(221)

c INCLUDE CONTRIBUTION to tau from NON-THERMAL scattering too

      do i=1,ngambin
        tauc(i) = 0.0
        do j=2,nebin
          tauc(i) = tauc(i) + (edist(j)+posdist(j))*compr(j,i)
        enddo
      enddo



      x = log10(theta)
c      print *,'ello theta  ',theta,edist(1),posdist(1)
c       edist(1) =0.
c       posdist(1)= 0.

c ** do escape!!
c ** Remember N for Tau_thomp = (ne- + ne+)=2!!!*edist(1)
         tau = 1./3.*(edist(1)+posdist(1))
         do 303 i = 1,ioff
           bbspect(i) = s(i)/(1.d0+tau+tauc(i))
303      continue
930    do 30 i = 1+ioff,81+ioff
         ii = i-ioff
         if (theta .lt. .15) then
           trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
         elseif ((theta .ge. .15).and.(theta .lt. .16)) then
           x1 = log10(.15)
           x2 = log10(.16)
           trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
           qrate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
           qrate=10.**qrate
           trate = trate+(qrate-trate)/(x2-x1)*(x-x1)
           trate=dmin1(1.d0,trate)
         elseif (theta .ge. .16d0) then
           trate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
           trate=10.**trate
           trate=dmin1(trate,1.d0)
         endif
         tau = 1./3.*trate*(edist(1)+posdist(1))
         bbspect(i) = s(i)/(1. + tau+tauc(i))
30     continue
       do 309 i =82+ioff,100+ioff
         ii = i - ioff
         if (theta .lt. .15) then
           trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
         elseif ((theta .ge. .15).and.(theta .lt. .16)) then
           x1 = log10(.15)
           x2 = log10(.16)
           trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
           qrate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
           qrate=10.**qrate
           trate = trate+(qrate-trate)/(x2-x1)*(x-x1)
           trate=dmin1(1.d0,trate)
         elseif (theta .ge. .16) then
           trate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
           trate=10.**trate
           trate=dmin1(trate,1.d0)
         endif
         tau = 1./2.7*trate*(edist(1)+posdist(1))*(1.-wgam(i))
         bbspect(i) = s(i)/(1. + tau+tauc(i))
309    continue
       do 399 i =101+ioff,181+ioff
         bbspect(i)= s(i)/(1.d0 + tauc(i))
399    continue
101    return
       end

c ----------------------------------------

       subroutine doesc
       include "common.eq"

      x = log10(theta)
      print *,'ello theta  ',theta,edist(1),posdist(1)
c       edist(1) =0.
c       posdist(1)= 0.

c ** do escape!!
c ** Remember N for Tau_thomp = (ne- + ne+)=2!!!*edist(1)
         tau = 1./3.*(edist(1)+posdist(1))
         do 303 i = 1,ioff
           pout(i) = pout(i)+1./(1.+tau)
           esc(i) = gdist(i)/(1.+tau)
303      continue
930    do 30 i = 1+ioff,81+ioff
         ii = i-ioff
         if (theta .lt. .15) then
           trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
         elseif ((theta .ge. .15).and.(theta .lt. .16)) then
           x1 = log10(.15)
           x2 = log10(.16)
           trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
           qrate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
           qrate=10.**qrate
           trate = trate+(qrate-trate)/(x2-x1)*(x-x1)
           trate=dmin1(1.d0,trate)
         elseif (theta .ge. .16d0) then
           trate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
           trate=10.**trate
           trate=dmin1(trate,1.d0)
         endif
         tau = 1./3.*trate*(edist(1)+posdist(1))
         pout(i) = pout(i)+1./(1.+tau)
         esc(i) = gdist(i)/(1. + tau)
30     continue
       do 309 i =82+ioff,100+ioff
         ii = i - ioff
         if (theta .lt. .15) then
           trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
         elseif ((theta .ge. .15).and.(theta .lt. .16)) then
           x1 = log10(.15)
           x2 = log10(.16)
           trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
           qrate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
           qrate=10.**qrate
           trate = trate+(qrate-trate)/(x2-x1)*(x-x1)
           trate=dmin1(1.d0,trate)
         elseif (theta .ge. .16) then
           trate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
           trate=10.**trate
           trate=dmin1(trate,1.d0)
         endif
         tau = 1./2.7*trate*(edist(1)+posdist(1))*(1.-wgam(i))
         pout(i) = pout(i)+1./(1.+tau)
         esc(i) = gdist(i)/(1. + tau)
309    continue
       do 399 i =101+ioff,181+ioff
         pout(i) = pout(i)+1.
         esc(i)= gdist(i)
399    continue
101    return
       end

       subroutine dotherm
       include "common.eq"


c -- START: check temperature first
c      if (theta .gt. .15) write(6,*) 'Warning theta too high!',theta
c      if (theta .lt. .001) write(6,*) 'Warning theta too low!',theta

      x = log10(theta)
c*********      IDIOTSKI DON'T FORGET
      eth=(1.d0+27.d0/8.d0*theta+45.d0/8.d0*theta*theta)
     &   /(1.d0+15.d0/8.d0*theta)*(edist(1)+posdist(1))

c --- annihilation
c   (first do annihilation of thermal bin with itself)
c use Svennson approx.
c
c zero out gder(i) so we can still thscat routine verbatim
       do 233 i=1,ngambin
233     gder(i) = 0.
  

       if (theta .lt. 0.005) then
         ga = 1./(1.+2.*theta*theta/log(1.120*theta+1.3))
         trate = ga*.375e0
         rate = trate*edist(1)*posdist(1)
	 if (edist(1) .gt. 1.e-12) then
           eav = eth/(edist(1)+posdist(1))
	 else
	   eav = 1+1.5*theta
	 endif
         disp = thann(3,1)*theta+thann(3,2)
         call thscat(2.d0*rate,eav,disp)
       else
c use Svensson (83)
        xl = (1.0-sqrt(3.0)/3.0)/2.0
        xu = (1.0+sqrt(3.0)/3.0)/2.0
        ttau = edist(1)*posdist(1)
        do 510 i = 80+ioff,142+ioff
          rlb = upwgam(i-1)
          wtarg = rlb + gwidth(i)*xl
          rl =  annspme(wtarg,theta)
          wtarg = rlb + gwidth(i)*xu
          ru = annspme(wtarg,theta)
c    AZ annspme gives answer for unit opt. depth     
c    if edist(1) = 1, => tau=2. AZ is factor 4 too small
          trate =  (ru+rl)*gwidth(i)
          rate =  trate*ttau*2.0
          gder(i) = gder(i) + rate
510     continue
      endif
      
      esum = 0
      do i=1,ngambin
        esum = esum + gder(i)*wgam(i)
      enddo


c ---- PAIR Escape!!!
c      (use AZ phenomenological fudge)
c      eder(1) = eder(1) - .3*edist(1)
c      ethder = ethder - .3*eth
c      eout(1) = eout(1) + .3

999   continue

c ** initialize escape array
c ** Remember N for Tau_thomp = (ne- + ne+)=2!!!*edist(1)
         tau = 1./3.*(edist(1)+posdist(1))
         do 303 i = 1,ioff
           tesc(i) = 1.d0/(1.d0+tau)
303      continue
930    do 30 i = 1+ioff,81+ioff
         ii = i-ioff
         if (theta .lt. .15) then
           trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
         elseif ((theta .ge. .15).and.(theta .lt. .16)) then
           x1 = log10(.15)
           x2 = log10(.16)
           trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
           qrate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
           qrate=10.**qrate
           trate = trate+(qrate-trate)/(x2-x1)*(x-x1)
           trate=dmin1(1.d0,trate)
         elseif (theta .ge. .16d0) then
           trate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
           trate=10.**trate
           trate=dmin1(trate,1.d0)
         endif
         tau = 1./3.*trate*(edist(1)+posdist(1))
         tesc(i) = 1./(1.+tau)
30     continue
       do 309 i =82+ioff,100+ioff
         ii = i - ioff
         if (theta .lt. .15) then
           trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
         elseif ((theta .ge. .15).and.(theta .lt. .16)) then
           x1 = log10(.15)
           x2 = log10(.16)
           trate =dmin1(cr(ii,1)*theta+cr(ii,2),1.d0)
           qrate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
           qrate=10.**qrate
           trate = trate+(qrate-trate)/(x2-x1)*(x-x1)
           trate=dmin1(1.d0,trate)
         elseif (theta .ge. .16) then
           trate=htcr(ii,1)+x*(htcr(ii,2)+x*(htcr(ii,3)+x*htcr(ii,4)))
           trate=10.**trate
           trate=dmin1(trate,1.d0)
         endif
         tau = 1./2.7*trate*(edist(1)+posdist(1))*(1.-wgam(i))
         tesc(i) = 1./(1.+tau)
309    continue
       do 399 i =101+ioff,181+ioff
         tesc(i)= 1.d0
399    continue
101    return
       end
      subroutine mkscat
      include "common.eq"
      dimension wi(15),a(6),b(6),ien(15),xt(6)
c * xi values
       data  a /0.222846604179,
     *          1.188932101673,
     *          2.992736326059,
     *          5.775143569105,
     *          9.837467418383,
     *         15.982873980602/
c * wi values   
        data b /4.58964673950e-1,
     *          4.17000830772e-1,
     *          1.13373382074e-1,
     *          1.03991974531e-2,
     *          2.61017202815e-4,
     *          8.98547906430e-7/

c
c --- compton scattering
c(Assume e- aren't scattered out of thermal dist.)
c --- Also do photon escape (only count thermal e+/e- in
c      determining escape rate. Escape rate formula is one
c      used by big Z.)

99     continue

       do 43 i=1,ngambin-20
         iscat(i,1) = ngambin
         iscat(i,2) = 1
         do 44 j=1,ngambin 
           scat(i,j) = 0.d0
44       continue
43     continue
       do i=ngambin-20,ngambin
         do j=1,ngambin
           scat(i,j)  = 0.d0
         enddo
       enddo


c *** Use SLEDGE hammer approach for everything, ie. the laguerre
c   integration fudge. (may be a bit of overkill, but we must get
c   rid of glitches und have Ordnung hier!)
c*** sledge hammer approach
        thint = 0.0
        do 189 lm = 1,6
          xp = a(lm)*theta + 1.0
          xt(lm) = xp
          if (xp .ge. 101.) then 
            ien(lm) = max(int(log10(xp)/.05+1.5),1)
          else
            ien(lm)=-max(int(log10(xp-1.0)/.05+101),1)
          endif
          wi(lm) = b(lm)*xp*sqrt(xp*xp-1.0)
          thint = thint + wi(lm)
189     continue
       do 190 lm=1,6
190      wi(lm)=wi(lm)/thint*(edist(1)+posdist(1))
      do 310 ii = 1,6
        i = ien(ii)     
        weight = wi(ii)
        if (i .gt. 0) then 
c ====     use non-thermal comp. data
c ignore photons > x=10^3
      do 320 j = 1,ngambin-ioff-20
        jj = j + ioff
        grate = ecomp(i,j,1)*weight
        pout(jj) = pout(jj) + grate
c Take care of scattering of PHOTONS into other bins
      i1 = igcomp(i,j,1)+ioff
      j1 = igcomp(i,j,2)+ioff
      if (i1 .lt. iscat(jj,1)) iscat(jj,1) = i1
      if (j1 .gt. iscat(jj,2)) iscat(jj,2) = j1
      dn1 = gcomp(i,j,2)
      rnp = gcomp(i,j,3)
      if (i1 .eq. j1) then
        scat(jj,i1) = scat(jj,i1) + grate
      elseif (abs(j1-i1) .eq. 1) then
        scat(jj,i1)= scat(jj,i1) + grate*dn1
        scat(jj,j1) = scat(jj,j1) + grate*(1. - dn1)
      else
        trnp = grate*rnp
        tdn1 = grate*dn1
        dw = trnp/(upwgam(j1-1) - upwgam(i1))
        do 340 k = (i1+1),(j1-1)
          scat(jj,k) = scat(jj,k) + dw*gwidth(k)
340      continue
        scat(jj,i1) = scat(jj,i1) + tdn1
        scat(jj,j1) = scat(jj,j1) + (grate  - tdn1) - trnp
      endif
320   continue
c  go do low energy comp. the same way
c  NB: All rates are assumed to be=sigma_t, since should be in
c       Thompson limit.
c  *** 80 should be ibot !!!! (in program with lcinit)
c achtung, yup, but knocked off two decades
      do 420 j = 1,40
        rate = weight
        dn1 = rldisp(i,j,1)
        rnp = rldisp(i,j,2)
        pout(j) = pout(j) + weight
        i1 = ig(i,j,1)
        j1 = ig(i,j,2)
	if (i1.lt.iscat(j,1)) iscat(j,1) = i1
        if (j1.gt.iscat(j,2)) iscat(j,2) = j1
        if (i1 .eq. j1) then
          scat(j,i1) = scat(j,i1) + rate
        elseif (abs(j1-i1) .eq. 1) then
          scat(j,i1)= scat(j,i1) + rate*dn1
          scat(j,j1) = scat(j,j1) + rate*(1. - dn1)
        else
          trnp = rate*rnp
          tdn1 = rate*dn1
          dw = trnp/(upwgam(j1-1) - upwgam(i1))
          do 490 k = (i1+1),(j1-1)
            scat(j,k) = scat(j,k) + dw*gwidth(k)
490        continue
          scat(j,i1) = scat(j,i1) + tdn1
          scat(j,j1) = scat(j,j1) + (rate  - tdn1) - trnp
        endif
420    continue
c       ** (end ians1 if, for locomp)
c == end of non-therm. data use
      else
c == use extended, low gamma data
       i = -i
       ip1 = min(i+1,141)
       if (i .eq. ip1) then
         frac = 0
       else
c BUGUGUGU!!!!!!!!!!!!!!!!!!!!!!!!!????????
c         frac=(xt(ii)-exn(ip1))/(exn(ip1)-exn(i))
c YES, idiotski!!! DOUBLE CHECK  
         frac=(xt(ii)-exn(i))/(exn(ip1)-exn(i))
       endif
       do 321 j=1,ngambin -20
         jc = j + 40
         trate=cltr(i,jc)+frac*(cltr(ip1,jc)-cltr(i,jc))
         eav  =clta(i,jc)+frac*(clta(ip1,jc)-clta(i,jc))
         tdisp=cltd(i,jc)+frac*(cltd(ip1,jc)-cltd(i,jc))
c         print *,j,trate,eav,tdisp
c         b2 = (xt(ii)*xt(ii)-1)/xt(ii)/xt(ii)
c         print *,'ap',wgam(j)*(1.+4./3.*b2-wgam(j))
c         test=test+eav*trate*weight/(edist(1)+posdist(1))
         grate = trate*weight
         pout(j) = pout(j) + grate
c go scatter!  ============================
         emin = eav-tdisp
         emax = eav+tdisp
         if (emax .lt. emin) then
           emin = eav
           emax = eav
         endif
         if (emin .lt. wmin) then 
           tdisp = abs(eav - wmin - 1.e-10)
           emax = eav+tdisp
           emin = eav - tdisp
         endif
         if (emax .gt. wmax) then
           tdisp = abs(wmax - eav - 1.e-10)
         endif
         emax = eav+tdisp
         emin = eav - tdisp
         i1 = max0(int(log10(emin)/.05 + rioff1),1)
         j1 = min0(int(log10(emax)/.05 + rioff1),ngambin)
c      do 777 klm =1,ngambin
c777     store2(klm) = gder(klm)

c -- changed from ge
      if (wgam(i1) .ge. eav) then
        i1 = i1 -1  
        if (i1 .lt. 1) then
          scat(j,1) = scat(j,1) + grate
          print *,'ERROR in thscat - eav < wgam(1)',eav
          goto 321
        endif
      endif
c -- changed from le
      if (wgam(j1) .le. eav) then
        j1 = j1 + 1
        if (j1 .gt. ngambin) then
          scat(j,ngambin)=scat(j,ngambin)+grate
          print *,'ERROR in thscat - eav > wgam(162)',eav
          goto 321
        endif
      endif
c      print *,'i1 = ',i1
c      print *,'j1 = ',j1
c      if (i1 .eq. j1) pause 'rat Foo!'
      if (i1.lt.iscat(j,1)) iscat(j,1) = i1
      if (j1.gt.iscat(j,2)) iscat(j,2) = j1
      if (abs(j1-i1) .eq. 1) then
        dn1 = grate*(eav - wgam(j1))/(wgam(i1) - wgam(j1))
        scat(j,i1)= scat(j,i1) + dn1
        scat(j,j1) = scat(j,j1) + grate - dn1
      else
        eavp = .5*(upwgam(i1)+ upwgam(j1-1))
        dw = 1./(emax-emin) 
c        if (eavp .eq. wgam(j1)) pause 'Here it is!'
        rm1 = (eav - wgam(j1))/(eavp - wgam(j1)) 
c        if (eavp .eq. wgam(i1)) pause 'no its here!'
        rm2 = (eav - wgam(i1))/(eavp - wgam(i1))
        ratdelt = upwgam(j1-1) - upwgam(i1)
        dw = min(dw*ratdelt,rm1,rm2)/ratdelt
        rnp = grate*ratdelt*dw
        do 30 k = (i1+1),(j1-1)
          scat(j,k) = scat(j,k) + grate*dw*(upwgam(k) - upwgam(k-1))
30      continue
        dn1 = grate*eav - rnp*eavp + wgam(j1)*(rnp - grate)
        dn1 = dn1/(wgam(i1)-wgam(j1))
        scat(j,i1) = scat(j,i1) + dn1
        scat(j,j1) = scat(j,j1) + (grate  - dn1) - rnp
      endif
c      ave = 0.
c      rpart = 0.
c      do 787 kk = 1,ngambin
c        rpart = rpart + gder(kk) - store2(kk)
c787     ave = ave + (gder(kk)- store2(kk))*wgam(kk)
c      print *,rpart,grate
c      print *,ave/rpart,eav
c      if (abs(rpart -grate)/grate .gt. 1.e-5) then
c        write(6,*) 'another horrid problem!'
c        write(6,*) i,j,'supposed',grate,'actual',rpart
c        write(6,*) 'i,j: ',i1,j1,eav,de2,targ,c,frac
c        pause 'LUnch!'
c      endif
c      if (rpart .ne. 0.) then
c        ave = ave / rpart
c        if (abs(ave -eav)/ave .gt. 1.e-5) then
c          write(6,*) 'we have a big problem!!!'
c          write(6,*) i,j,i1,j1,'ave',ave,'eav',eav
c          pause 'CRUNCH!'
c        endif
c      endif
c ========================
321    continue
       endif
310    continue


       do 343 i=1,ngambin-20
          pout(i) = pout(i) - scat(i,i)
          scat(i,i) =  0.0
c          dwscat(i) = 0.
c          do j=iscat(i,1),iscat(i,2)
c             dwscat(i) = scat(i,j)*(wgam(i)-wgam(j)) + dwscat(i)
c          enddo
343    continue

         return
         end
      subroutine getethd
      include "common.eq"
      dimension wi(15),a(6),b(6),ien(15),xt(6)
c * xi values
       data  a /0.222846604179,
     *          1.188932101673,
     *          2.992736326059,
     *          5.775143569105,
     *          9.837467418383,
     *         15.982873980602/
c * wi values   
        data b /4.58964673950e-1,
     *          4.17000830772e-1,
     *          1.13373382074e-1,
     *          1.03991974531e-2,
     *          2.61017202815e-4,
     *          8.98547906430e-7/

      x = log10(theta)


c********* 	IDIOTSKI DON'T FORGET
      eth=(1.d0+27.d0/8.d0*theta+45.d0/8.d0*theta*theta)
     &   /(1.d0+15.d0/8.d0*theta)*(edist(1)+posdist(1))

      tethder = ethder

c --- annihilation
c   (first do annihilation of thermal bin with itself)
c use Svennson approx.
c
c *** remember ethder is TOTAL change in thermal energy 
       eout2(1) = 0.
       if (theta .lt. 0.005) then
         ga = 1./(1.+2.*theta*theta/log(1.120*theta+1.3))
         trate = ga*.375e0
         rate = trate*edist(1)*posdist(1)
	 if (edist(1) .gt. 1.e-12) then
           eav = eth/(edist(1)+posdist(1))
	 else
	   eav = 1+1.5*theta
	 endif
         eout2(1) = eout2(1) + trate
         ethder = ethder - rate*eav*2.
       else
c use Svensson (83)
        xl = (1.0-sqrt(3.0)/3.0)/2.0
        xu = (1.0+sqrt(3.0)/3.0)/2.0
        ttau = edist(1)*posdist(1)
        do 510 i = 80+ioff,142+ioff
          rlb = upwgam(i-1)
          wtarg = rlb + gwidth(i)*xl
          rl =  annspme(wtarg,theta)
          wtarg = rlb + gwidth(i)*xu
          ru = annspme(wtarg,theta)
c    AZ annspme gives answer for unit opt. depth     
c    if edist(1) = 1, => tau=2. AZ is factor 4 too small
          trate =  (ru+rl)*gwidth(i)
          rate =  trate*ttau*2.0
          eout2(1) = eout2(1) + trate
c  ATTENTO! ACHTUNG -- Gefahr
          ethder = ethder - rate*wgam(i)
510     continue
      endif
c debug
      ga = 1./(1.+2.*theta*theta/log(1.120*theta+1.3))
      zrate = ga*.375e0
      zrate = zrate
      tethder = ethder



c
c --- compton scattering
c(Assume e- aren't scattered out of thermal dist.)
c --- Also do photon escape (only count thermal e+/e- in
c      determining escape rate. Escape rate formula is one
c      used by big Z.)

99     continue

c *** Use SLEDGE hammer approach for everything, ie. the laguerre
c   integration fudge. (may be a bit of overkill, but we must get
c   rid of glitches und have Ordnung hier!)
c*** sledge hammer approach
        thint = 0.0
        do 189 lm = 1,6
          xp = a(lm)*theta + 1.0
          xt(lm) = xp
          if (xp .ge. 101.) then 
            ien(lm) = max(int(log10(xp)/.05+1.5),1)
          else
            ien(lm)=-max(int(log10(xp-1.0)/.05+101),1)
          endif
          wi(lm) = b(lm)*xp*sqrt(xp*xp-1.0)
          thint = thint + wi(lm)
189     continue
       do 190 lm=1,6
190      wi(lm)=wi(lm)/thint*(edist(1)+posdist(1))
      do 310 ii = 1,6
        i = ien(ii)     
        weight = wi(ii)
        if (i .gt. 0) then 
c ====     use non-thermal comp. data
c ignore photons > x=10^3
      do 320 j = 1,ngambin-ioff-20
        jj = j + ioff
        grate = ecomp(i,j,1)*weight*gdist(jj)
        ethder = ethder + (wgam(jj)-av(i,jj))*grate
320   continue
c  go do low energy comp. the same way
c  NB: All rates are assumed to be=sigma_t, since should be in
c       Thompson limit.
c  *** 80 should be ibot !!!! (in program with lcinit)
c achtung, yup, but knocked off two decades
      do 420 j = 1,40
        rate = weight*gdist(j)
        ethder = ethder + (wgam(j)-clav(i,j))*grate
420    continue
c       ** (end ians1 if, for locomp)
c == end of non-therm. data use
      else
c == use extended, low gamma data
       i = -i
       ip1 = min(i+1,141)
       if (i .eq. ip1) then
         frac = 0
       else
c BUGUGUGU!!!!!!!!!!!!!!!!!!!!!!!!!????????
c         frac=(xt(ii)-exn(ip1))/(exn(ip1)-exn(i))
c YES, idiotski!!! DOUBLE CHECK
         frac=(xt(ii)-exn(i))/(exn(ip1)-exn(i))
       endif
       do 321 j=1,ngambin -20
         jc = j + 40
         trate=cltr(i,jc)+frac*(cltr(ip1,jc)-cltr(i,jc))
         eav  =clta(i,jc)+frac*(clta(ip1,jc)-clta(i,jc))
         grate = trate*weight*gdist(j)
         ethder = ethder + (wgam(j)-eav)*grate
321    continue
       endif
310    continue

         return
         end

c============================
      subroutine doelec
      include "common.eq" 
      real*8 escat(81,81),eout0(81),posout0(81),tescat(81,81)
      external dtime
      real dtime
      real time(2)

c      start = dtime(time)

c      goto 232
c get sources
c  -----------  pair production

      superoth = theta
      isth = 0

       
      psum = 0.
      do 110 i = 141,221
      do 120 j = 1,i
        ii = i-140
        if (ppr(ii,j).lt. .005) goto 120
        if (ipp(ii,j) .lt. 1) goto 120
        erate = gdist(i)*gdist(j)*ppr(ii,j)
c        if ((erate .lt. rmin) .or. (erate .lt. 1.e-9)) goto 99
        i1 = ipp(ii,j)
        i2 = i1 + 1
	frac = ppf(ii,j)
        tfr = frac*erate
        ein(i1) = ein(i1) + tfr
        ein(i2) = ein(i2) + erate - tfr
        posin(i1) = posin(i1) + tfr
        posin(i2) = posin(i2) + erate-tfr
        psum = psum + erate
120   continue
110   continue

c get thomson optical depth
      call getth(1)


c     setup scattering matrix
      index = 0
      do i=1,81
        do j=1,81
          escat(i,j) = 0.0
        enddo
      enddo


c for my scheme
        i=1
        do j=1,221
          do k=iectab(i,j,1),iectab(i,j,2)
            index=index+1
          enddo
        enddo
 
      do i=2,81
        do j=1,221
          do k=iectab(i,j,1),iectab(i,j,2)
            index=index+1
            escat(k,i) =escat(k,i)+ectab(index)*gdist(j)
          enddo
        enddo
      enddo

      do i=2,81
        do j=1,221
          if(compr(i,j).gt.0) eout(i) = eout(i) + compr(i,j)*gdist(j)
        enddo
        eout(i) = eout(i) - escat(i,i)
        escat(i,i) = 0.0
        posout(i) = eout(i)
      enddo

      do i=1,nebin
        ein0(i) = ein(i)
        posin0(i) = posin(i)
        eout0(i) = eout(i)
        posout0(i) = posout(i)
      enddo
      do  i = 2,nebin
         tescat(i-1,i) = escat(i-1,i)
      enddo

1     continue

c always call bremsstrahlung, and if are doing thermal distribution,docoul
c REMEMBER to zero out GDOT!
      do i=1,nebin
        gdot(i) = 0.
      enddo
      call dobrem
      if (itherm .gt. 0) then
c turn on cooling smoothly or else, boom!
        call docoul
      endif

c-- convert gdots to derivatives (eder)
c  dumb, first order advection scheme which explicitly conserves
c  energy.
       do 43 i = 2,nebin
         if (gdot(i) .le. 0.) then
               frac = gdot(i)/deltag(i)
               eout(i) =  eout(i) -frac
               posout(i) = posout(i) - frac
               escat(i-1,i) = escat(i-1,i) - frac
         elseif (i .lt. nebin) then
           pause 'positive gdot?'
           frac = gdot(i)/deltag(i+1)
           eout(i) = eout(i) + frac
           escat(i+1,i)=escat(i+1,i) + frac
         endif
43     continue

920   continue
 
      do i=1,81
        ein(i) = ein0(i)
        posin(i) = posin0(i)
        do j=2,81
          ein(i) = ein(i)+escat(i,j)*edist(j)
          posin(i) = posin(i) + escat(i,j)*posdist(j)
        enddo
      enddo
 
      dmax = 0
      do i=2,81
        if (eout(i).ne.0.d0) then 
            temp = ein(i)/eout(i)
            dmax = max(abs(temp-edist(i))/(edist(i)+1.d-40),dmax)
            edist(i) = temp
        endif
        if (posout(i).ne.0.d0) then
           temp = posin(i)/posout(i)
           dmax = max(abs(temp-posdist(i))/(posdist(i)+1.d-40),dmax)
           posdist(i) = temp
        endif
      enddo

      if (dmax.gt. 1.d-4) goto 920

      call getth(1)
      dth = abs(theta-superoth)/superoth
      superoth = theta
c      print *,'.....................dth',dth

      isth=isth + 1
      if ((dth .lt. 1.d-2)) goto 928
      do i=1,nebin
        ein(i) = ein0(i)
        posin(i) = posin0(i)
        eout(i) = eout0(i)
        posout(i) = posout0(i)
      enddo
      do  i = 2,nebin
         escat(i-1,i) = tescat(i-1,i)
      enddo

      goto 1

928   continue

c      stop = dtime(time)
c      print *,'e- time=',time(1)

      return
      end

      real*8 function annspme(x, temp)
      implicit real*8 (a-h,o-z)
c     This program calculates the annihilation spectrum from a thermal
c     pure pair plasma of unit Thomson depth using the formulae by
c     Svensson 1983.
c     x and temp are in units of m c**2. The result gives photon number
c     spectrum in units of sigma(Thomson) c m c**2/ m c**2
      pi=3.14159
      a1=(x/temp)**2 /2 /(1+2.0049/temp +1.4774/temp/temp
     >    +pi/(2*temp)**3)
      y=x*temp
      if(y.le.4.) then
          a2=pi/2 *sqrt(pi/y) *exp((2-x-1/x)/temp)
          if(y.le.0.25) then
              y4=4*y
              cc=1+0.5288*y4-0.4483*y4**2+0.1643*y4**3
          else if(y.le.1.) then
              cc=1.125+0.6600*y-0.7972*y**2+0.3060*y**3
          else
              cc=0.9049+1.1613/y-1.2487/y**2+0.4770/y**3
          end if
      else
          a2=pi/y*exp((2-x)/temp)*(log(4*0.56146*y)-1)
          y4=4/y
          cc=1+0.5289*y4-0.8254*y4**2+0.9811*y4**3-0.3895*y4**4
      end if
c  3/32/pi follow from assuming n+=n- and using sigma(Thomson)
      annspme=a1*a2*cc/temp*3/32./pi
      return
      end

c***********************************************************************
      SUBROUTINE ASPLINE(X,Y,N,YP1,YPN,Y2)
      IMPLICIT REAL*8(A-H,O-Z)
      PARAMETER (NMAX=500)
      DIMENSION X(N),Y(N),Y2(N),U(NMAX)
      IF (YP1.GT..99D30) THEN
        Y2(1)=0.
        U(1)=0.
      ELSE
        Y2(1)=-0.5
        U(1)=(3./(X(2)-X(1)))*((Y(2)-Y(1))/(X(2)-X(1))-YP1)
      ENDIF
      DO 11 I=2,N-1
        SIG=(X(I)-X(I-1))/(X(I+1)-X(I-1))
        P=SIG*Y2(I-1)+2.
        Y2(I)=(SIG-1.)/P
        U(I)=(6.*((Y(I+1)-Y(I))/(X(I+1)-X(I))-(Y(I)-Y(I-1))
     *      /(X(I)-X(I-1)))/(X(I+1)-X(I-1))-SIG*U(I-1))/P
11    CONTINUE
      IF (YPN.GT..99D30) THEN
        QN=0.
        UN=0.
      ELSE
        QN=0.5
        UN=(3./(X(N)-X(N-1)))*(YPN-(Y(N)-Y(N-1))/(X(N)-X(N-1)))
      ENDIF
      Y2(N)=(UN-QN*U(N-1))/(QN*Y2(N-1)+1.)
      DO 12 K=N-1,1,-1
        Y2(K)=Y2(K)*Y2(K+1)+U(K)
12    CONTINUE
      RETURN
      END
 
 
      SUBROUTINE ASPLINT(XA,YA,Y2A,N,X,Y)
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION XA(N),YA(N),Y2A(N)
      KLO=1
      KHI=N
1     IF (KHI-KLO.GT.1) THEN
        K=(KHI+KLO)/2
        IF(XA(K).GT.X)THEN
          KHI=K
        ELSE
          KLO=K
        ENDIF
      GOTO 1
      ENDIF
      H=XA(KHI)-XA(KLO)
      IF (H.EQ.0.) PAUSE 'Bad XA input.'
      A=(XA(KHI)-X)/H
      B=(X-XA(KLO))/H
      Y=A*YA(KLO)+B*YA(KHI)+
     *      ((A**3-A)*Y2A(KLO)+(B**3-B)*Y2A(KHI))*(H**2)/6.
      RETURN
      END

c*********************************************************************
c     calculates Compton reflected spectrum
c*********************************************************************
      subroutine comprefl1(x,compsp,xr,refl,angle,
     + par_refl,nrefl,jmax,jref,irefl)
c     compsp - angle averaged comptonized spectrum (flux)
c     xr - energies (1:jref) where refl should be computed
c     refl  - reflected spectrum (intensities) at a given angle
c     angle - cos of angle 
      PARAMETER(nem=5000)
      implicit real*8 (a-h,o-z)
      integer jmax,jref,nmin,nmax,i,irefl
      real*4 spref(nem),xref(nem),spc1
      real*4 gamma,ab_,ab0,temp,xi,xm,ec
      real*8 x(jmax),xr(jref),compsp(jmax),refl(jref),angle
      real*4 cmin,par_refl(nrefl)
      real*8 XLOG(nem),DSPXX(nem)
      data cmin/1e-30/
c alog(1 keV/511) alog(10/511)
      data XMIN/-6.236/,XMAX/-3.934/,ec/0./
      external spc1

c Fe abudance
      ab0=par_refl(1)
c  ab_       - abundance of elements heavier than He relative to
c              the solar abundances
      ab_=par_refl(2)
c  xi        - disk ionization parameter = L/nR^2
      xi=par_refl(3)
c  temp      - disk temperature (for ionization balance, Tmax=10^6 K)
      temp=par_refl(4)

      do i=1,jref
      xref(i)=sngl(xr(i))
      enddo

c put here calculation of spline for input spectrum
c Don't forget to convert spectrum to Juri "flux" units
      do 212 i=1,jmax
      dspxx(i)=dlog(compsp(i)+cmin)
      xlog(i)=dlog(x(i))
  212 continue
c      CALL QSPLINE(XLOG,DSPXX,JMAX,1d30,1d30,F2)

c compute here photon spectral slope gamma in interval 1-10 keV
      CALL ALOCATE(XLOG,JMAX,XMIN,NMIN)
      NMIN=NMIN+1
      CALL ALOCATE(XLOG,JMAX,XMAX,NMAX)
      IF(NMIN.ge.NMAX) NMIN=NMAX-1
      CALL ALININT(DSPXX,XLOG,A,B1,NMIN,NMAX,JMAX)
      gamma=-sngl(A)

      xm=angle
c call compton reflection for a given angle
c spref gives  x F_x
c static reflection
      if(irefl.eq.0) then
      call greeion(xref,spref,xm,ab_,ab0,spc1,gamma,ec,temp,xi,jref)
c relativistic disk reflection
      else
      call relrefl(xref,spref,xm,gamma,spc1, 
     + par_refl,nrefl,jref,1) 
      endif

       do ix=1,jref
c intensity = flux/cos_angle
c ** BUT I WANT flux, in  contrast to juri
c divide by one more ix since I want number flux
       refl(ix)=dble(spref(ix))/xr(ix)/xr(ix)
       enddo

c      print *,'finished comprefl'
      return
      end


c ===============================
c For Pavel's Reflection!
c compute P(Y) = N(Y)*Y^(-2) = N(1/E)*(1/E)^-2 = N(1/E)*E^2
c E and Y are measured in units of M_eC^2
      real*4 function spc1(y,gamma,ec)

      include "common.eq" 
      real*4 y,gamma,ec
      common/compsp1/ ospect(221)
c interpolation 
      targ = 1.d0/y
      i = max(min(int(log10(targ)*20.d0+141.5d0),221),1)
      if (targ.lt.wgam(i))  i=max(i-1,1)
      i2 = min(221,i+1)
      x0 = wgam(i)
      x1 = wgam(i2)
      y1 = ospect(i2)
      y0 = ospect(i)
      if (i.ne.i2) then 
        spect = y0*exp(log(y1/y0)/log(x1/x0)*log(targ/x0))
      else
        spect = y0
      endif
      spc1 = spect/y/y
      return
      end

c**************************************************
      SUBROUTINE ALOCATE(XX,N,X,J)
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION XX(N)
      JL=0
      JU=N+1
10    IF(JU-JL.GT.1)THEN
        JM=(JU+JL)/2
        IF((XX(N).GT.XX(1)).EQV.(X.GT.XX(JM)))THEN
          JL=JM
        ELSE
          JU=JM
        ENDIF
      GO TO 10
      ENDIF
      J=JL
      RETURN
      END
c**************************************************
C linear interpolation:
C abciss:  XX
C ordinate: FUNLOG
C A,B: coefficients of the linear function Ax+B
      SUBROUTINE ALININT(FUNLOG,XLOG,A,B,NMIN,NMAX,II)
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION FUNLOG(II),XLOG(II)
      DATA ERR/1D30/
      IF(NMAX.le.NMIN) THEN
      A=ERR
      B=ERR
      ELSE
      S=NMAX-NMIN+1
      SX=0.
      SY=0.
      SXX=0.
      SXY=0.
           DO 15 IX=NMIN,NMAX
           X=XLOG(IX)
           DI=FUNLOG(IX)
           SX=SX+X
           SY=SY+DI
           SXX=SXX+X*X
           SXY=SXY+DI*X
   15      CONTINUE
      DELTA=S*SXX-SX*SX
      B=(SXX*SY-SX*SXY)/DELTA
      A=(S*SXY-SX*SY)/DELTA
      ENDIF
      RETURN
      END
 


      SUBROUTINE gauleg(x1,x2,x,w,n)
      implicit real*8 (a-h,o-z)
      INTEGER n
      REAL*8 x1,x2,x(n),w(n)
      DOUBLE PRECISION EPS
      PARAMETER (EPS=3.d-14)
      INTEGER i,j,m
      DOUBLE PRECISION p1,p2,p3,pp,xl,xm,z,z1
      m=(n+1)/2
      xm=0.5d0*(x2+x1)
      xl=0.5d0*(x2-x1)
      do 12 i=1,m
        z=cos(3.141592654d0*(i-.25d0)/(n+.5d0))
1       continue
          p1=1.d0
          p2=0.d0
          do 11 j=1,n
            p3=p2
            p2=p1
            p1=((2.d0*j-1.d0)*z*p2-(j-1.d0)*p3)/j
11        continue
          pp=n*(z*p1-p2)/(z*z-1.d0)
          z1=z
          z=z1-p1/pp
        if(abs(z-z1).gt.EPS)goto 1
        x(i)=xm-xl*z
        x(n+1-i)=xm+xl*z
        w(i)=2.d0*xl/((1.d0-z*z)*pp*pp)
        w(n+1-i)=w(i)
12    continue
      return
      END


